##############################################################################
#
# Copyright (c) 2005 Zope Corporation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Test form i18n

$Id: test_functional_i18n.py 81040 2007-10-24 15:27:12Z srichter $
"""

import re
import unittest
from persistent import Persistent
from zope.testing import renormalizing, doctest
from zope.interface import Interface, implements
from zope.schema import TextLine, Text, Int, List
from zope.i18nmessageid import MessageFactory
from zope.app.testing.functional import FunctionalDocFileSuite
from zope.app.form.testing import AppFormLayer


_ = MessageFactory('formtest')

__docformat__ = "reStructuredText"


class IFieldContent(Interface):

    title = TextLine(
        title=_(u"Title"),
        description=_(u"A short description of the event."),
        default=u"",
        required=True
        )

    description = Text(
        title=_(u"Description"),
        description=_(u"A long description of the event."),
        default=u"",
        required=False
        )

    somenumber = Int(
        title=_(u"Some number"),
        default=0,
        required=False
        )

    somelist = List(
        title=_(u"Some List"),
        value_type=TextLine(title=_(u"Some item")),
        default=[],
        required=False
        )


class FieldContent(Persistent):
    implements(IFieldContent)


checker = renormalizing.RENormalizing([
    (re.compile(r"HTTP/1\.1 200 .*"), "HTTP/1.1 200 OK"),
    ])


def test_suite():
    i18n = FunctionalDocFileSuite('i18n.txt', package='zope.app.form.browser',
        checker=checker)
    i18n.layer = AppFormLayer
    return unittest.TestSuite([
        i18n,
        ])


if __name__ == '__main__':
    unittest.main(defaultTest='test_suite')



##############################################################################
#
# Copyright (c) 2001, 2002 Zope Corporation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Radio Widget Functional Tests

$Id: test_functional_booleanradiowidget.py 81040 2007-10-24 15:27:12Z srichter $
"""
import unittest
import transaction
from persistent import Persistent

import zope.security.checker
from zope.interface import Interface, implements
from zope.schema import Bool
from zope.traversing.api import traverse

from zope.app.form.testing import AppFormLayer
from zope.app.form.browser.tests.support import *
from zope.app.testing.functional import BrowserTestCase

class IFoo(Interface):

    bar = Bool(title=u'Bar')

class Foo(Persistent):

    implements(IFoo)

    def __init__(self):
        self.bar = True

class Test(BrowserTestCase):

    def setUp(self):
        BrowserTestCase.setUp(self)
        registerEditForm(IFoo, widgets={
            'bar': { 'class': 'zope.app.form.browser.BooleanRadioWidget' }})
        defineSecurity(Foo, IFoo)

    def test_display_editform(self):
        self.getRootFolder()['foo'] = Foo()
        transaction.commit()

        # display edit view
        response = self.publish('/foo/edit.html')
        self.assertEqual(response.getStatus(), 200)

        # bar field should be displayed as two radio buttons
        self.assert_(patternExists(
            '<input .*checked="checked".*name="field.bar".*type="radio".*'
            'value="on".* />',
            response.getBody()))
        self.assert_(patternExists(
            '<input .*name="field.bar".*type="radio".*value="off".* />',
            response.getBody()))

        # a hidden element is used to note that the field is present
        self.assert_(patternExists(
            '<input name="field.bar-empty-marker" type="hidden" value="1".* />',
            response.getBody()))


    def test_submit_editform(self):
        self.getRootFolder()['foo'] = Foo()
        transaction.commit()

        # submit edit view
        response = self.publish('/foo/edit.html', form={
            'UPDATE_SUBMIT' : '',
            'field.bar' : 'off'})
        self.assertEqual(response.getStatus(), 200)
        self.assert_(updatedMsgExists(response.getBody()))

        # check new values in object
        object = traverse(self.getRootFolder(), 'foo')
        self.assertEqual(object.bar, False)


    def test_missing_value(self):
        self.getRootFolder()['foo'] = Foo()
        transaction.commit()

        # temporarily make bar field not required
        IFoo['bar'].required = False

        # submit missing value for bar
        response = self.publish('/foo/edit.html', form={
            'UPDATE_SUBMIT' : '',
            'field.bar-empty-marker' : '' })
        self.assertEqual(response.getStatus(), 200)
        self.assert_(updatedMsgExists(response.getBody()))

        # confirm use of missing_value as new object value
        self.assert_(IFoo['bar'].missing_value is None)
        object = traverse(self.getRootFolder(), 'foo')
        self.assert_(object.bar is None)

        # restore bar required state
        IFoo['bar'].required = True


    def test_required_validation(self):
        self.getRootFolder()['foo'] = Foo()
        transaction.commit()

        self.assert_(IFoo['bar'].required)

        # submit missing value for required field bar
        response = self.publish('/foo/edit.html', form={
            'UPDATE_SUBMIT' : '',
            'field.bar-empty-marker' : '1'})
        self.assertEqual(response.getStatus(), 200)

        # confirm error msgs
        self.assert_(missingInputErrorExists('bar', response.getBody()))


    def test_invalid_allowed_value(self):
        self.getRootFolder()['foo'] = Foo()
        transaction.commit()

        # submit a value for bar isn't allowed
        response = self.publish('/foo/edit.html', form={
            'UPDATE_SUBMIT' : '',
            'field.bar' : 'bogus' })
        self.assertEqual(response.getStatus(), 200)

        self.assert_(validationErrorExists('bar', 'Invalid value',
            response.getBody()))


def test_suite():
    suite = unittest.TestSuite()
    Test.layer = AppFormLayer
    suite.addTest(unittest.makeSuite(Test))
    return suite

if __name__=='__main__':
    unittest.main(defaultTest='test_suite')



##############################################################################
#
# Copyright (c) 2001, 2002 Zope Corporation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Integer Widget Tests

$Id: test_intwidget.py 69218 2006-07-20 04:12:28Z baijum $
"""
import unittest
from zope.testing import doctest
from unittest import main, makeSuite
from zope.app.form.browser.tests.test_browserwidget import SimpleInputWidgetTest
from zope.app.form.interfaces import IInputWidget
from zope.app.form.browser import IntWidget
from zope.app.form.interfaces import ConversionError, WidgetInputError
from zope.interface.verify import verifyClass

from zope.schema import Int


class IntWidgetTest(SimpleInputWidgetTest):
    """Documents and tests the int widget.

        >>> verifyClass(IInputWidget, IntWidget)
        True
    """

    _FieldFactory = Int
    _WidgetFactory = IntWidget

    def test_hasInput(self):
        del self._widget.request.form['field.foo']
        self.failIf(self._widget.hasInput())
        # widget has input, even if input is an empty string
        self._widget.request.form['field.foo'] = u''
        self.failUnless(self._widget.hasInput())
        self._widget.request.form['field.foo'] = u'123'
        self.failUnless(self._widget.hasInput())

    def test_getInputValue(self):
        self._widget.request.form['field.foo'] = u''
        self.assertRaises(WidgetInputError, self._widget.getInputValue)
        self._widget.request.form['field.foo'] = u'123'
        self.assertEquals(self._widget.getInputValue(), 123)
        self._widget.request.form['field.foo'] = u'abc'
        self.assertRaises(ConversionError, self._widget.getInputValue)


def test_suite():
    return unittest.TestSuite((
        unittest.makeSuite(IntWidgetTest),
        doctest.DocTestSuite(),
        ))

if __name__=='__main__':
    unittest.main(defaultTest='test_suite')



##############################################################################
#
# Copyright (c) 2003 Zope Corporation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""General test support.

$Id: support.py 81040 2007-10-24 15:27:12Z srichter $
"""
import re
from zope.configuration import xmlconfig

class VerifyResults(object):
    """Mix-in for test classes with helpers for checking string data."""

    def verifyResult(self, result, check_list, inorder=False):
        start = 0
        for check in check_list:
            pos = result.find(check, start)
            self.assert_(pos >= 0,
                         "%r not found in %r" % (check, result[start:]))
            if inorder:
                start = pos + len(check)

    def verifyResultMissing(self, result, check_list):
        for check in check_list:
            self.assert_(result.find(check) < 0,
                         "%r unexpectedly found in %r" % (check, result))

def registerEditForm(schema, widgets={}):
    """Registers an edit form for the specified schema.

    widgets is a mapping of field name to dict. The dict for each field must
    contain a 'class' item, which is the widget class, and any additional
    widget attributes (e.g. text field size, rows, cols, etc.)
    """
    widgetsXml = []
    for field in widgets:
        widgetsXml.append('<widget field="%s"' % field)
        for attr in widgets[field]:
            widgetsXml.append(' %s="%s"' % (attr, widgets[field][attr]))
        widgetsXml.append(' />')
    xmlconfig.string("""
        <configure xmlns="http://namespaces.zope.org/browser">
          <include package="zope.app.form.browser" file="meta.zcml" />
          <editform
            name="edit.html"
            schema="%s"
            permission="zope.View">
            %s
          </editform>
        </configure>
        """ % (schema.__identifier__, ''.join(widgetsXml)))


def defineSecurity(class_, schema):
    class_ = '%s.%s' % (class_.__module__, class_.__name__)
    schema = schema.__identifier__
    xmlconfig.string("""
        <configure xmlns="http://namespaces.zope.org/zope">
          <include package="zope.app.component" file="meta.zcml" />
          <class class="%s">
            <require
              permission="zope.Public"
              interface="%s"
              set_schema="%s" />
          </class>
        </configure>
        """ % (class_, schema, schema))


def defineWidgetView(field_interface, widget_class, view_type):
    field_interface = field_interface.__identifier__
    widget_class = '%s.%s' % (widget_class.__module__, widget_class.__name__)
    view_type = '%s.%s' % (view_type.__module__, view_type.__name__)
    xmlconfig.string("""
        <configure xmlns="http://namespaces.zope.org/zope">
          <include package="zope.app.component" file="meta.zcml" />
          <view
            for="%s"
            type="zope.publisher.interfaces.browser.IBrowserRequest"
            factory="%s"
            provides="%s"
            permission="zope.Public"
            />
        </configure>
        """ % (field_interface, widget_class, view_type))


def patternExists(pattern, source, flags=0):
    return re.search(pattern, source, flags) is not None


def validationErrorExists(field, error_msg, source):
    regex = re.compile(r'%s.*?name="field.(\w+)(?:\.[\w\.]+)?"' % (error_msg,),
                       re.DOTALL)
    # compile it first because Python 2.3 doesn't allow flags in findall
    return field in regex.findall(source)


def missingInputErrorExists(field, source):
    return validationErrorExists(field, 'Required input is missing.', source)


def invalidValueErrorExists(field, source):
    # assumes this error is displayed for select elements
    return patternExists(
        'Invalid value.*name="field.%s".*</select>' % field,
        source, re.DOTALL)


def updatedMsgExists(source):
    return patternExists('<p>Updated .*</p>', source)



##############################################################################
#
# Copyright (c) 2001, 2002 Zope Corporation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Support for display-only pages based on schema.

$Id: schemadisplay.py 67630 2006-04-27 00:54:03Z jim $
"""
__docformat__ = 'restructuredtext'

import zope.component
from zope.interface import Interface
from zope.publisher.interfaces.browser import IDefaultBrowserLayer
from zope.publisher.browser import BrowserView
from zope.schema import getFieldNamesInOrder
from zope.security.checker import defineChecker, NamesChecker

from zope.app.form.utility import setUpDisplayWidgets
from zope.app.pagetemplate.viewpagetemplatefile import ViewPageTemplateFile
from zope.app.pagetemplate.simpleviewclass import SimpleViewClass

class DisplayView(BrowserView):
    """Simple display-view base class.

    Subclasses should provide a `schema` attribute defining the schema
    to be displayed.
    """

    errors = ()
    update_status = ''
    label = ''

    # Fall-back field names computes from schema
    fieldNames = property(lambda self: getFieldNamesInOrder(self.schema))

    def __init__(self, context, request):
        super(DisplayView, self).__init__(context, request)
        self._setUpWidgets()

    def _setUpWidgets(self):
        self.adapted = self.schema(self.context)
        setUpDisplayWidgets(self, self.schema, source=self.adapted,
                            names=self.fieldNames)

    def setPrefix(self, prefix):
        for widget in self.widgets():
            widget.setPrefix(prefix)

    def widgets(self):
        return [getattr(self, name+'_widget')
                for name in self.fieldNames]


def DisplayViewFactory(name, schema, label, permission, layer,
                       template, default_template, bases, for_, fields,
                       fulledit_path=None, fulledit_label=None):
    class_ = SimpleViewClass(template, used_for=schema, bases=bases,
                             name=name)
    class_.schema = schema
    class_.label = label
    class_.fieldNames = fields
    class_.fulledit_path = fulledit_path
    if fulledit_path and (fulledit_label is None):
        fulledit_label = "Full display"
    class_.fulledit_label = fulledit_label
    class_.generated_form = ViewPageTemplateFile(default_template)
    defineChecker(class_,
                  NamesChecker(("__call__", "__getitem__", "browserDefault"),
                               permission))

    if layer is None:
        layer = IDefaultBrowserLayer

    sm = zope.component.getGlobalSiteManager()
    sm.registerAdapter(class_, (for_, layer), Interface, name)



##############################################################################
#
# Copyright (c) 2004 Zope Corporation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Test widget registrations.

$Id: test_registrations.py 73522 2007-03-24 23:51:08Z fdrake $
"""
import unittest

from zope.component import getMultiAdapter
from zope.configuration import xmlconfig
from zope.interface import implements
from zope.publisher.browser import TestRequest
from zope.testing.doctestunit import DocTestSuite

from zope.app.testing import setup
# import all widgets (in this case, importing * is ok, since we
# absolutely know what we're importing)
from zope.app.form.browser import *

from zope.app.form.interfaces import IDisplayWidget, IInputWidget
import zope.app.form.browser

import zope.schema as fields
from zope.schema import interfaces
from zope.schema import vocabulary

class ISampleObject(interfaces.IField):
    pass

class SampleObject(object):
    implements(ISampleObject)

class ISampleVocabulary(interfaces.IVocabularyTokenized,
                        interfaces.IVocabulary):
    pass

class SampleVocabulary(vocabulary.SimpleVocabulary):
    implements(ISampleVocabulary)

request = TestRequest()
sample = SampleObject()
vocab = SampleVocabulary([])

def setUp(test):
    setup.placelessSetUp()
    context = xmlconfig.file("tests/registerWidgets.zcml",
                             zope.app.form.browser)

class Tests(object):
    """Documents and tests widgets registration for specific field types.
    
    Standard Widgets
    ------------------------------------------------------------------------
    The relationships between field types and standard widgets are listed
    below.
    
    IField, IDisplayWidget -> DisplayWidget
        
        >>> field = fields.Field()
        >>> widget = getMultiAdapter((field, request), IDisplayWidget)
        >>> isinstance(widget, DisplayWidget)
        True
        
    ITextLine, IInputWidget -> TextWidget 
        
        >>> field = fields.TextLine()
        >>> widget = getMultiAdapter((field, request), IInputWidget)
        >>> isinstance(widget, TextWidget)
        True
        
    IText, IInputWidget -> TextAreaWidget
    
        >>> field = fields.Text()
        >>> widget = getMultiAdapter((field, request), IInputWidget)
        >>> isinstance(widget, TextAreaWidget)
        True
        
    ISourceText, IInputWidget -> TextAreaWidget
    
        >>> field = fields.SourceText()
        >>> widget = getMultiAdapter((field, request), IInputWidget)
        >>> isinstance(widget, TextAreaWidget)
        True

    IBytesLine, IInputWidget -> BytesWidget
    
        >>> field = fields.BytesLine()
        >>> widget = getMultiAdapter((field, request), IInputWidget)
        >>> isinstance(widget, BytesWidget)
        True

    IBytes, IInputWidget -> FileWidget
    
        >>> field = fields.Bytes()
        >>> widget = getMultiAdapter((field, request), IInputWidget)
        >>> isinstance(widget, FileWidget)
        True
        		
    IASCIILine, IInputWidget -> ASCIIWidget
    
        >>> field = fields.ASCIILine()
        >>> widget = getMultiAdapter((field, request), IInputWidget)
        >>> isinstance(widget, ASCIIWidget)
        True
        
    IASCII, IInputWidget -> ASCIIAreaWidget
    
        >>> field = fields.ASCII()
        >>> widget = getMultiAdapter((field, request), IInputWidget)
        >>> isinstance(widget, ASCIIAreaWidget)
        True
        
    IInt, IInputWidget -> IntWidget
    
        >>> field = fields.Int()
        >>> widget = getMultiAdapter((field, request), IInputWidget)
        >>> isinstance(widget, IntWidget)
        True
        
    IFloat, IInputWidget -> FloatWidget
    
        >>> field = fields.Float()
        >>> widget = getMultiAdapter((field, request), IInputWidget)
        >>> isinstance(widget, FloatWidget)
        True

    IDecimal, IInputWidget -> DecimalWidget
    
        >>> field = fields.Decimal()
        >>> widget = getMultiAdapter((field, request), IInputWidget)
        >>> isinstance(widget, DecimalWidget)
        True
        
    IDatetime, IInputWidget -> DatetimeWidget
    
        >>> field = fields.Datetime()
        >>> widget = getMultiAdapter((field, request), IInputWidget)
        >>> isinstance(widget, DatetimeWidget)
        True
        
    IDate, IInputWidget -> DateWidget
    
        >>> field = fields.Date()
        >>> widget = getMultiAdapter((field, request), IInputWidget)
        >>> isinstance(widget, DateWidget)
        True
        
    IBool, IInputWidget -> CheckBoxWidget
    
        >>> field = fields.Bool()
        >>> widget = getMultiAdapter((field, request), IInputWidget)
        >>> isinstance(widget, CheckBoxWidget)
        True
        
    ITuple, IInputWidget -> TupleSequenceWidget
    
        >>> field = fields.Tuple(value_type=fields.Int())
        >>> widget = getMultiAdapter((field, request), IInputWidget)
        >>> isinstance(widget, TupleSequenceWidget)
        True

    IList, IInputWidget -> ListSequenceWidget
    
        >>> field = fields.List(value_type=fields.Int())
        >>> widget = getMultiAdapter((field, request), IInputWidget)
        >>> isinstance(widget, ListSequenceWidget)
        True

    IPassword, IInputWidget -> PasswordWidget
    
        >>> field = fields.Password()
        >>> widget = getMultiAdapter((field, request), IInputWidget)
        >>> isinstance(widget, PasswordWidget)
        True

    IChoice, IDisplayWidget -> ItemDisplayWidget
    
        >>> field = fields.Choice(vocabulary=vocab)
        >>> field = field.bind(sample)
        >>> widget = getMultiAdapter((field, request), IDisplayWidget)
        >>> isinstance(widget, ItemDisplayWidget)
        True
                
    IChoice, IInputWidget -> DropdownWidget
    
        >>> field = fields.Choice(vocabulary=vocab)
        >>> field = field.bind(sample)
        >>> widget = getMultiAdapter((field, request), IInputWidget)
        >>> isinstance(widget, DropdownWidget)
        True

    IList with IChoice value_type, IDisplayWidget -> ItemsMultiDisplayWidget
    
        >>> field = fields.List(value_type=fields.Choice(vocabulary=vocab))
        >>> field = field.bind(sample)
        >>> widget = getMultiAdapter((field, request), IDisplayWidget)
        >>> isinstance(widget, ItemsMultiDisplayWidget)
        True
                
    IList with IChoice value_type, IInputWidget -> MultiSelectWidget
    
        >>> field = fields.List(value_type=fields.Choice(vocabulary=vocab))
        >>> field = field.bind(sample)
        >>> widget = getMultiAdapter((field, request), IInputWidget)
        >>> isinstance(widget, OrderedMultiSelectWidget)
        True
    """

def test_suite():    
    return DocTestSuite(setUp=setUp, tearDown=setup.placelessTearDown)

if __name__=='__main__':
    unittest.main(defaultTest='test_suite')



##############################################################################
#
# Copyright (c) 2001, 2002 Zope Corporation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Checkbox Widget tests

$Id: test_checkboxwidget.py 69613 2006-08-17 15:55:52Z fdrake $
"""
import unittest
from zope.testing import doctest
from zope.app.form.interfaces import IInputWidget
from zope.app.form.browser import CheckBoxWidget
from zope.publisher.browser import TestRequest
from zope.schema import Bool
from zope.interface.verify import verifyClass

from zope.app.form.interfaces import MissingInputError
from zope.app.form.browser.tests.test_browserwidget import SimpleInputWidgetTest


class CheckBoxWidgetTest(SimpleInputWidgetTest):
    """Documents and tests thec checkbox widget.

        >>> verifyClass(IInputWidget, CheckBoxWidget)
        True

    The checkbox widget works with Bool fields:

        >>> field = Bool(__name__='foo', title=u'on')
        >>> request = TestRequest()
        >>> widget = CheckBoxWidget(field, request)

    hasInput returns True when the request contains the field.<name>.used
    value:

        >>> 'field.foo.used' in request.form
        False
        >>> widget.hasInput()
        False
        >>> request.form['field.foo.used'] = ''
        >>> widget.hasInput()
        True

    getInputValue returns True when field.<name> equals (and only equals) 'on':

        >>> 'field.foo' in request.form
        False
        >>> widget.getInputValue()
        False
        >>> request.form['field.foo'] = 'true'
        >>> widget.getInputValue()
        False
        >>> request.form['field.foo'] = 'on'
        >>> widget.getInputValue()
        True

    Below is HTML output of rendered checkbox widgets. We will first define
    a helper method condense the HTML output for display in this test:

        >>> def normalize(s):
        ...   return '\\n  '.join(s.split())

    Default widget rendering:

        >>> print normalize( widget() )
        <input
          class="hiddenType"
          id="field.foo.used"
          name="field.foo.used"
          type="hidden"
          value=""
          />
          <input
          class="checkboxType"
          checked="checked"
          id="field.foo"
          name="field.foo"
          type="checkbox"
          value="on"
          />

    Hidden rendering:

        >>> print normalize( widget.hidden() )
        <input
          class="hiddenType"
          id="field.foo"
          name="field.foo"
          type="hidden"
          value="on"
          />

    Calling setRenderedValue will change what gets output:

        >>> widget.setRenderedValue(False)
        >>> print normalize( widget() )
        <input
          class="hiddenType"
          id="field.foo.used"
          name="field.foo.used"
          type="hidden"
          value=""
          />
          <input
          class="checkboxType"
          id="field.foo"
          name="field.foo"
          type="checkbox"
          value="on"
          />

    The checkbox widget does not support None values, so a Bool required
    constraint will always be met with checkbox input:

        >>> field.required = True
        >>> widget.getInputValue()
        True
    """

    _FieldFactory = Bool
    _WidgetFactory = CheckBoxWidget

    def testProperties(self):
        self.assertEqual(self._widget.tag, 'input')
        self.assertEqual(self._widget.type, 'checkbox')
        self.assertEqual(self._widget.cssClass, '')
        self.assertEqual(self._widget.extra, '')
        self.assertEqual(self._widget.default, 0)

    def testRender(self):
        value = 1
        self._widget.setRenderedValue(value)
        check_list = ('type="checkbox"', 'id="field.foo"',
                      'name="field.foo"', 'checked="checked"')
        self.verifyResult(self._widget(), check_list)
        value = 0
        self._widget.setRenderedValue(value)
        check_list = check_list[:-1]
        self.verifyResult(self._widget(), check_list)
        check_list = ('type="hidden"',) + check_list[1:-1]
        self.verifyResult(self._widget.hidden(), check_list)
        check_list = ('style="color: red"',) + check_list
        self._widget.extra = 'style="color: red"'
        self.verifyResult(self._widget.hidden(), check_list)

    def test_getInputValue(self):
        self._widget.request.form['field.foo'] = 'on'
        self.assertEqual(self._widget.getInputValue(), True)
        self._widget.request.form['field.foo'] = 'positive'
        self.assertEqual(self._widget.getInputValue(), False)
        del self._widget.request.form['field.foo']
        self._widget.request.form['field.foo.used'] = ''
        self.assertEquals(self._widget.getInputValue(), False)
        del self._widget.request.form['field.foo.used']
        self.assertRaises(MissingInputError, self._widget.getInputValue)

    def test_required(self):
        # checkbox widgets are never required, since there's no way to
        # set it to "no value"
        self.failIf(self._widget.required)
        self.assert_(self._widget.context.required)


def test_suite():
    return unittest.TestSuite((
        unittest.makeSuite(CheckBoxWidgetTest),
        doctest.DocTestSuite(),
        ))

if __name__=='__main__':
    unittest.main(defaultTest='test_suite')



##############################################################################
#
# Copyright (c) 2001, 2002 Zope Corporation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Float Widget Functional Tests

$Id: test_functional_floatwidget.py 81040 2007-10-24 15:27:12Z srichter $
"""
import unittest
import transaction
from persistent import Persistent

import zope.security.checker
from zope.interface import Interface, implements
from zope.traversing.api import traverse
from zope.schema import Float, Choice

from zope.app.form.testing import AppFormLayer
from zope.app.form.browser.tests.support import *
from zope.app.testing.functional import BrowserTestCase

class IFloatTest(Interface):

    f1 = Float(
        required=False,
        min=1.1,
        max=10.1)

    f2 = Float(
        required=False)

    f3 = Choice(
        required=True,
        values=(0.0, 1.1, 2.1, 3.1, 5.1, 7.1, 11.1),
        missing_value=0)


class FloatTest(Persistent):

    implements(IFloatTest)

    def __init__(self):
        self.f1 = None
        self.f2 = 1.1
        self.f3 = 2.1


class Test(BrowserTestCase):

    def setUp(self):
        BrowserTestCase.setUp(self)
        registerEditForm(IFloatTest)
        defineSecurity(FloatTest, IFloatTest)

    def test_display_editform(self):
        self.getRootFolder()['test'] = FloatTest()
        transaction.commit()

        # display edit view
        response = self.publish('/test/edit.html')
        self.assertEqual(response.getStatus(), 200)

        # f1 and f2 should be displayed in text fields
        self.assert_(patternExists(
            '<input .* name="field.f1".* value="".*>', response.getBody()))
        self.assert_(patternExists(
            '<input .* name="field.f2".* value="1.1".*>', response.getBody()))

        # f3 should be in a dropdown
        self.assert_(patternExists(
            '<select .*name="field.f3".*>', response.getBody()))
        self.assert_(patternExists(
            '<option selected="selected" value="2.1">2.1</option>',
            response.getBody()))


    def test_submit_editform(self):
        self.getRootFolder()['test'] = FloatTest()
        transaction.commit()

        # submit edit view
        response = self.publish('/test/edit.html', form={
            'UPDATE_SUBMIT' : '',
            'field.f1' : '1.123',
            'field.f2' : '2.23456789012345',
            'field.f3' : '11.1' })
        self.assertEqual(response.getStatus(), 200)
        self.assert_(updatedMsgExists(response.getBody()))

        # check new values in object
        object = traverse(self.getRootFolder(), 'test')
        self.assertEqual(object.f1, 1.123)
        self.assertEqual(object.f2, 2.23456789012345)
        self.assertEqual(object.f3, 11.1)


    def test_missing_value(self):
        self.getRootFolder()['test'] = FloatTest()
        transaction.commit()

        # submit missing values for f2 and f3
        response = self.publish('/test/edit.html', form={
            'UPDATE_SUBMIT' : '',
            'field.f1' : '',
            'field.f2' : '',
            'field.f3' : '1.1' })
        self.assertEqual(response.getStatus(), 200)
        self.assert_(updatedMsgExists(response.getBody()))

        # check new values in object
        object = traverse(self.getRootFolder(), 'test')
        self.assertEqual(object.f1, None)
        self.assertEqual(object.f2, None) # None is default missing_value
        self.assertEqual(object.f3, 1.1)  # 0 is from f3.missing_value=0


    def test_required_validation(self):
        self.getRootFolder()['test'] = FloatTest()
        transaction.commit()

        # submit missing values for required field f1
        response = self.publish('/test/edit.html', form={
            'UPDATE_SUBMIT' : '',
            'field.f1' : '',
            'field.f2' : '',
            'field.f3' : '' })
        self.assertEqual(response.getStatus(), 200)

        # confirm error msgs
        self.assert_(not missingInputErrorExists('f1', response.getBody()))
        self.assert_(not missingInputErrorExists('f2', response.getBody()))
        self.assert_(missingInputErrorExists('f3', response.getBody()))


    def test_invalid_allowed_value(self):
        self.getRootFolder()['test'] = FloatTest()
        transaction.commit()

        # submit a value for f3 that isn't allowed
        response = self.publish('/test/edit.html', form={
            'UPDATE_SUBMIT' : '',
            'field.f3' : '10000' })
        self.assertEqual(response.getStatus(), 200)
        self.assert_(invalidValueErrorExists('f3', response.getBody()))


    def test_min_max_validation(self):
        self.getRootFolder()['test'] = FloatTest()
        transaction.commit()

        # submit value for f1 that is too low
        response = self.publish('/test/edit.html', form={
            'UPDATE_SUBMIT' : '',
            'field.f1' : '-1' })
        self.assertEqual(response.getStatus(), 200)
        self.assert_(validationErrorExists('f1', 'Value is too small',
            response.getBody()))

        # submit value for f1 that is too high
        response = self.publish('/test/edit.html', form={
            'UPDATE_SUBMIT' : '',
            'field.f1' : '1000.2' })
        self.assertEqual(response.getStatus(), 200)
        self.assert_(validationErrorExists('f1', 'Value is too big',
            response.getBody()))


    def test_omitted_value(self):
        self.getRootFolder()['test'] = FloatTest()
        transaction.commit()

        # confirm default values
        object = traverse(self.getRootFolder(), 'test')
        self.assert_(object.f1 is None)
        self.assertEqual(object.f2, 1.1)
        self.assertEqual(object.f3, 2.1)

        # submit change with only f2 present -- note that required
        # field f1 is omitted, which should not cause a validation error
        response = self.publish('/test/edit.html', form={
            'UPDATE_SUBMIT' : '',
            'field.f2' : '' })
        self.assertEqual(response.getStatus(), 200)
        self.assert_(updatedMsgExists(response.getBody()))

        # check new value in object
        object = traverse(self.getRootFolder(), 'test')
        self.assert_(object.f1 is None)
        self.assert_(object.f2 is None)
        self.assertEqual(object.f3, 2.1)


    def test_conversion(self):
        self.getRootFolder()['test'] = FloatTest()
        transaction.commit()

        # submit value for f1 that cannot be convert to an float
        response = self.publish('/test/edit.html', form={
            'UPDATE_SUBMIT' : '',
            'field.f1' : 'foo' })
        self.assertEqual(response.getStatus(), 200)
        self.assert_(validationErrorExists(
            'f1',
            'Invalid floating point data', response.getBody()))


def test_suite():
    suite = unittest.TestSuite()
    Test.layer = AppFormLayer
    suite.addTest(unittest.makeSuite(Test))
    return suite

if __name__=='__main__':
    unittest.main(defaultTest='test_suite')




##############################################################################
#
# Copyright (c) 2001, 2002 Zope Corporation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""File Widget Tests

$Id: test_functional_filewidget.py 81040 2007-10-24 15:27:12Z srichter $
"""
import unittest
import transaction
from StringIO import StringIO
from persistent import Persistent

import zope.security.checker
from zope.interface import Interface, implements
from zope.schema.interfaces import IField
from zope.schema import Field
from zope.traversing.api import traverse

from zope.app.form.testing import AppFormLayer
from zope.app.form.browser.textwidgets import FileWidget
from zope.app.form.browser.tests.support import *
from zope.app.testing.functional import BrowserTestCase
from zope.app.form.interfaces import IInputWidget

class IFileField(IField):
    """Field for representing a file that can be edited by FileWidget."""


class FileField(Field):

    implements(IFileField)

class IFileTest(Interface):

    f1 = FileField(required=True)
    f2 = FileField(required=False)

class FileTest(Persistent):

    implements(IFileTest)

    def __init__(self):
        self.f1 = None
        self.f2 = 'foo'


class SampleTextFile(StringIO):

    def __init__(self, buf, filename=''):
        StringIO.__init__(self, buf)
        self.filename = filename


class Test(BrowserTestCase):

    sampleText = "The quick brown fox\njumped over the lazy dog."
    sampleTextFile = SampleTextFile(sampleText)

    emptyFileName = 'empty.txt'
    emptyFile = SampleTextFile('', emptyFileName)

    def setUp(self):
        BrowserTestCase.setUp(self)
        defineWidgetView(IFileField, FileWidget, IInputWidget)
        registerEditForm(IFileTest)
        defineSecurity(FileTest, IFileTest)

    def test_display_editform(self):
        self.getRootFolder()['test'] = FileTest()
        transaction.commit()

        # display edit view
        response = self.publish('/test/edit.html')
        self.assertEqual(response.getStatus(), 200)

        # field should be displayed in a file input element
        self.assert_(patternExists(
            '<input .* name="field.f1".* type="file".*>', response.getBody()))
        self.assert_(patternExists(
            '<input .* name="field.f2".* type="file".*>', response.getBody()))


    def test_submit_text(self):
        self.getRootFolder()['test'] = FileTest()
        transaction.commit()
        object = traverse(self.getRootFolder(), 'test')
        self.assert_(object.f1 is None)
        self.assertEqual(object.f2, 'foo')

        # submit a sample text file
        response = self.publish('/test/edit.html', form={
            'UPDATE_SUBMIT' : '',
            'field.f1' : self.sampleTextFile,
            'field.f2' : self.sampleTextFile,
            'field.f1.used' : '',
            'field.f2.used' : '' })
        self.assertEqual(response.getStatus(), 200)
        self.assert_(updatedMsgExists(response.getBody()))

        # check new values in object
        object = traverse(self.getRootFolder(), 'test')
        self.assertEqual(object.f1, self.sampleText)
        self.assertEqual(object.f2, self.sampleText)


    def test_invalid_value(self):
        self.getRootFolder()['test'] = FileTest()
        transaction.commit()

        # submit an invalid file value
        response = self.publish('/test/edit.html', form={
            'UPDATE_SUBMIT' : '',
            'field.f1' : 'not a file - same as missing input',
            'field.f1.used' : '',
            'field.f2.used' : '' })
        self.assertEqual(response.getStatus(), 200)
        self.assert_(validationErrorExists('f1',
            'Form input is not a file object', response.getBody()))


    def test_required_validation(self):
        self.getRootFolder()['test'] = FileTest()
        transaction.commit()

        # submit missing value for required field f1
        response = self.publish('/test/edit.html', form={
            'UPDATE_SUBMIT' : '',
            'field.f1.used' : '',
            'field.f2.used' : ''})
        self.assertEqual(response.getStatus(), 200)

        # confirm error msgs
        self.assert_(missingInputErrorExists('f1', response.getBody()))
        self.assert_(not missingInputErrorExists('f2', response.getBody()))


    def test_empty_file(self):
        self.getRootFolder()['test'] = FileTest()
        transaction.commit()

        # submit an empty text file
        response = self.publish('/test/edit.html', form={
            'UPDATE_SUBMIT' : '',
            'field.f2' : self.emptyFile,
            # 'field.f1.used' : '', # we don't let f1 know that it was rendered
            # or else it will complain (see test_required_validation) and the
            # change will not succeed.
            'field.f2.used' : ''})
        self.assertEqual(response.getStatus(), 200)
        self.assert_(updatedMsgExists(response.getBody()))

        # new value for f1 should be field.missing_value (i.e, None)
        object = traverse(self.getRootFolder(), 'test')
        self.assert_(object.f1 is None)


def test_suite():
    suite = unittest.TestSuite()
    Test.layer = AppFormLayer
    suite.addTest(unittest.makeSuite(Test))
    return suite

if __name__=='__main__':
    unittest.main(defaultTest='test_suite')





##############################################################################
#
# Copyright (c) 2004 Zope Corporation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Browser widgets

$Id: __init__.py 73058 2007-03-08 09:49:48Z zagy $
"""
__docformat__ = 'restructuredtext'

from zope.app.form.browser.widget import BrowserWidget, DisplayWidget
from zope.app.form.browser.widget import UnicodeDisplayWidget

from zope.app.form.browser.textwidgets import TextWidget, BytesWidget
from zope.app.form.browser.textwidgets import TextAreaWidget, BytesAreaWidget
from zope.app.form.browser.textwidgets import PasswordWidget, FileWidget
from zope.app.form.browser.textwidgets import ASCIIWidget, ASCIIAreaWidget
from zope.app.form.browser.textwidgets import IntWidget, FloatWidget
from zope.app.form.browser.textwidgets import DecimalWidget
from zope.app.form.browser.textwidgets import DatetimeWidget, DateWidget
from zope.app.form.browser.textwidgets import DatetimeI18nWidget
from zope.app.form.browser.textwidgets import DateI18nWidget
from zope.app.form.browser.textwidgets import DatetimeDisplayWidget
from zope.app.form.browser.textwidgets import DateDisplayWidget
from zope.app.form.browser.textwidgets import BytesDisplayWidget
from zope.app.form.browser.textwidgets import ASCIIDisplayWidget
from zope.app.form.browser.textwidgets import URIDisplayWidget

# Widgets for boolean fields
from zope.app.form.browser.boolwidgets import CheckBoxWidget
from zope.app.form.browser.boolwidgets import BooleanRadioWidget
from zope.app.form.browser.boolwidgets import BooleanSelectWidget
from zope.app.form.browser.boolwidgets import BooleanDropdownWidget

# Choice and Sequence Display Widgets
from zope.app.form.browser.itemswidgets import ItemDisplayWidget
from zope.app.form.browser.itemswidgets import ItemsMultiDisplayWidget
from zope.app.form.browser.itemswidgets import SetDisplayWidget
from zope.app.form.browser.itemswidgets import ListDisplayWidget

# Widgets for fields with vocabularies.
# Note that these are only dispatchers for the widgets below.
from zope.app.form.browser.itemswidgets import ChoiceDisplayWidget
from zope.app.form.browser.itemswidgets import ChoiceInputWidget
from zope.app.form.browser.itemswidgets import CollectionDisplayWidget
from zope.app.form.browser.itemswidgets import CollectionInputWidget
from zope.app.form.browser.itemswidgets import ChoiceCollectionDisplayWidget
from zope.app.form.browser.itemswidgets import ChoiceCollectionInputWidget

# Widgets that let you choose a single item from a list
# These widgets are multi-views on (field, vocabulary)
from zope.app.form.browser.itemswidgets import SelectWidget
from zope.app.form.browser.itemswidgets import DropdownWidget
from zope.app.form.browser.itemswidgets import RadioWidget

# Widgets that let you choose several items from a list
# These widgets are multi-views on (field, vocabulary)
from zope.app.form.browser.itemswidgets import MultiSelectWidget
from zope.app.form.browser.itemswidgets import MultiSelectSetWidget
from zope.app.form.browser.itemswidgets import MultiSelectFrozenSetWidget
from zope.app.form.browser.itemswidgets import MultiCheckBoxWidget
from zope.app.form.browser.itemswidgets import OrderedMultiSelectWidget

# Widgets that let you enter several items in a sequence
# These widgets are multi-views on (sequence type, value type)
from zope.app.form.browser.sequencewidget import SequenceWidget
from zope.app.form.browser.sequencewidget import TupleSequenceWidget
from zope.app.form.browser.sequencewidget import ListSequenceWidget
from zope.app.form.browser.sequencewidget import SequenceDisplayWidget

from zope.app.form.browser.objectwidget import ObjectWidget



##############################################################################
#
# Copyright (c) 2002 Zope Corporation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Validation Exceptions

$Id: interfaces.py 97648 2009-03-08 08:24:24Z nadako $
"""
__docformat__ = 'restructuredtext'

from zope.schema.interfaces import ValidationError
from zope.publisher.interfaces import IView
from zope.interface import Attribute, Interface, implements
from zope.schema import Bool
from zope.exceptions.interfaces import UserError

class IWidgetInputError(Interface):
    """Placeholder for a snippet View"""

    def doc():
        """Returns a string that represents the error message."""

class WidgetInputError(UserError):
    """One or more user input errors occurred."""

    implements(IWidgetInputError)

    def __init__(self, field_name, widget_title, errors=None):
        """Initialize Error

        `errors` is a ``ValidationError`` or a list of ValidationError objects
        """
        UserError.__init__(self, field_name, widget_title, errors)
        self.field_name = field_name
        self.widget_title = widget_title
        self.errors = errors

    def doc(self):
        # TODO this duck typing is to get the code working.  See
        # collector issue 372
        if isinstance(self.errors, basestring):
            return self.errors
        elif getattr(self.errors, 'doc', None) is not None:
            return self.errors.doc()
        return ''


class MissingInputError(WidgetInputError):
    """Required data was not supplied."""


class ConversionError(Exception):
    """A conversion error occurred."""

    implements(IWidgetInputError)

    def __init__(self, error_name, original_exception=None):
        Exception.__init__(self, error_name, original_exception)
        self.error_name = error_name
        self.original_exception = original_exception

    def doc(self):
        return self.error_name

InputErrors = WidgetInputError, ValidationError, ConversionError


class ErrorContainer(Exception):
    """A base error class for collecting multiple errors."""

    def append(self, error):
        self.args += (error, )

    def __len__(self):
        return len(self.args)

    def __iter__(self):
        return iter(self.args)

    def __getitem__(self, i):
        return self.args[i]

    def __str__(self):
        return "\n".join(
            ["%s: %s" % (error.__class__.__name__, error)
             for error in self.args]
            )

    __repr__ = __str__

class WidgetsError(ErrorContainer):
    """A collection of errors from widget processing.

    widgetValues is a map containing the list of values that were obtained
    from the widgets, keyed by field name.
    """

    def __init__(self, errors, widgetsData={}):
        ErrorContainer.__init__(self, *errors)
        self.widgetsData = widgetsData

class IWidget(IView):
    """Generically describes the behavior of a widget.

    Note that this level must be still presentation independent.
    """

    name = Attribute(
        """The unique widget name

        This must be unique within a set of widgets.""")

    label = Attribute(
        """The widget label.

        Label may be translated for the request.

        The attribute may be implemented as either a read-write or read-only
        property, depending on the requirements for a specific implementation.

        """)

    hint = Attribute(
        """A hint regarding the use of the widget.

        Hints are traditionally rendered using tooltips in GUIs, but may be
        rendered differently depending on the UI implementation.

        Hint may be translated for the request.

        The attribute may be implemented as either a read-write or read-only
        property, depending on the requirements for a specific implementation.

        """)

    visible = Attribute(
        """A flag indicating whether or not the widget is visible.""")

    def setRenderedValue(value):
        """Set the value to be rendered by the widget.

        Calling this method will override any values provided by the user.

        For input widgets (`IInputWidget` implementations), calling
        this sets the value that will be rendered even if there is
        already user input.

        """

    def setPrefix(prefix):
        """Set the name prefix used for the widget

        The widget name is used to identify the widget's data within
        input data.  For example, for HTTP forms, the widget name is
        used for the form key.

        It is acceptable to *reset* the prefix: set it once to read
        values from the request, and again to redraw with a different
        prefix but maintained state.

        """

class IInputWidget(IWidget):
    """A widget for editing a field value."""

    required = Bool(
        title=u"Required",
        description=u"""If True, widget should be displayed as requiring input.

        By default, this value is the field's 'required' attribute. This
        field can be set to False for widgets that always provide input (e.g.
        a checkbox) to avoid unnecessary 'required' UI notations.
        """)

    def getInputValue():
        """Return value suitable for the widget's field.

        The widget must return a value that can be legally assigned to
        its bound field or otherwise raise ``WidgetInputError``.

        The return value is not affected by `setRenderedValue()`.
        """

    def applyChanges(content):
        """Validate the user input data and apply it to the content.

        Return a boolean indicating whether a change was actually applied.

        This raises an error if there is no user input.
        """

    def hasInput():
        """Returns ``True`` if the widget has input.

        Input is used by the widget to calculate an 'input value', which is
        a value that can be legally assigned to a field.

        Note that the widget may return ``True``, indicating it has input, but
        still be unable to return a value from `getInputValue`. Use
        `hasValidInput` to determine whether or not `getInputValue` will return
        a valid value.

        A widget that does not have input should generally not be used
        to update its bound field.  Values set using
        `setRenderedValue()` do not count as user input.

        A widget that has been rendered into a form which has been
        submitted must report that it has input.  If the form
        containing the widget has not been submitted, the widget
        shall report that it has no input.

        """

    def hasValidInput():
        """Returns ``True`` is the widget has valid input.

        This method is similar to `hasInput` but it also confirms that the
        input provided by the user can be converted to a valid field value
        based on the field constraints.
        """

class IDisplayWidget(IWidget):
    """A widget for displaying a field value."""

    required = Bool(
        title=u"Required",
        description=u"""If True, widget should be displayed as requiring input.

        Display widgets should never be required.
        """)

class IWidgetFactory(Interface):
    """A factory that creates the widget"""
    
    def __call__(context, request):
        """Return a widget"""



##############################################################################
#
# Copyright (c) 2001, 2002 Zope Corporation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Generic Text Widgets tests

$Id: test_displaywidget.py 70826 2006-10-20 03:41:16Z baijum $
"""
import unittest
from zope.interface.verify import verifyClass
from zope.interface.exceptions import DoesNotImplement
from zope.publisher.browser import TestRequest
from zope.schema import TextLine
from zope.testing.doctestunit import DocTestSuite

from zope.app.form.browser.widget import DisplayWidget, UnicodeDisplayWidget


def test_implemented_interfaces():
    """Make sure that the display widget implements the correct interfaces.

    Like all browser-used widgets, DisplayWidget must implement
    `IBrowserWidget`.

    >>> from zope.app.form.browser.interfaces import IBrowserWidget
    >>> verifyClass(IBrowserWidget, DisplayWidget)
    True

    But unlike most other widgets in this package, the display widget is *not*
    an `IInputWidget`.

    >>> from zope.app.form.interfaces import IInputWidget
    >>> try:
    ...     verifyClass(IInputWidget, DisplayWidget)    
    ... except DoesNotImplement:
    ...     'not implemented'
    'not implemented'
    """

def test_not_required():
    """Make sure that display widgets are not required

    >>> field = TextLine(title = u'Title',
    ...                  __name__ = u'title',
    ...                  default = u'<My Title>')
    >>> widget = DisplayWidget(field, TestRequest())
    >>> widget.required
    False
    
    """

def test_value_escaping():
    """Make sure that the returned values are correctly escaped.

    First we need to create a field that is the context of the display widget.
    >>> field = TextLine(title = u'Title',
    ...                  __name__ = u'title',
    ...                  default = u'<My Title>')

    >>> field = field.bind(None)

    Now we are ready to instantiate our widget.

    >>> widget = DisplayWidget(field, TestRequest())

    If no data was specified in the widget, the field's default value will be
    chosen.

    >>> widget()
    u'&lt;My Title&gt;'

    Now let's set a value and make sure that, when output, it is also
    correctly escaped.

    >>> widget.setRenderedValue(u'<Another Title>')
    >>> widget()
    u'&lt;Another Title&gt;'

    When the value is the missing_value, the empty string should be
    displayed::

    >>> field = TextLine(title = u'Title',
    ...                  __name__ = u'title',
    ...                  required = False)

    >>> field = field.bind(None)
    >>> widget = DisplayWidget(field, TestRequest())
    >>> widget.setRenderedValue(field.missing_value)

    >>> widget()
    ''

    If there's no default for the field and the value is missing on
    the bound object, the empty string should still be displayed::

    >>> field = TextLine(title=u'Title',
    ...                  __name__=u'title',
    ...                  required=False)

    >>> class Thing:
    ...    title = field.missing_value

    >>> field = field.bind(Thing())
    >>> widget = DisplayWidget(field, TestRequest())

    >>> widget()
    ''

    """


def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(DocTestSuite())
    suite.addTest(DocTestSuite(
        extraglobs={"DisplayWidget": UnicodeDisplayWidget}))
    return suite

if __name__ == '__main__':
    unittest.main(defaultTest='test_suite')



##############################################################################
#
# Copyright (c) 2001, 2002 Zope Corporation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""TextArea Functional Tests

$Id: test_functional_textareawidget.py 81040 2007-10-24 15:27:12Z srichter $
"""
import unittest
import transaction
from persistent import Persistent

import zope.security.checker
from zope.interface import Interface, implements
from zope.schema import Text
from zope.traversing.api import traverse

from zope.app.form.testing import AppFormLayer
from zope.app.form.browser.tests.support import *
from zope.app.testing.functional import BrowserTestCase

class ITextTest(Interface):

    s2 = Text(
        required=False,
        missing_value=u'')

    s3 = Text(
        required=False)

    s1 = Text(
        required=True,
        min_length=2,
        max_length=10)


class TextTest(Persistent):

    implements(ITextTest)

    def __init__(self):
        self.s1 = ''
        self.s2 = u'foo'
        self.s3 = None


class Test(BrowserTestCase):

    def setUp(self):
        BrowserTestCase.setUp(self)
        registerEditForm(ITextTest)
        defineSecurity(TextTest, ITextTest)

    def test_display_editform(self):
        self.getRootFolder()['test'] = TextTest()
        transaction.commit()

        # display edit view
        response = self.publish('/test/edit.html')
        self.assertEqual(response.getStatus(), 200)

        # all fields should be displayed in text fields
        self.assert_(patternExists(
            '<textarea .* name="field.s1".*></textarea>',
            response.getBody()))
        self.assert_(patternExists(
            '<textarea .* name="field.s2".*>foo</textarea>',
            response.getBody()))
        self.assert_(patternExists(
            '<textarea .* name="field.s3".*></textarea>',
            response.getBody()))


    def test_submit_editform(self):
        self.getRootFolder()['test'] = TextTest()
        transaction.commit()

        # submit edit view
        response = self.publish('/test/edit.html', form={
            'UPDATE_SUBMIT' : '',
            'field.s1' : u'foo',
            'field.s2' : u'bar',
            'field.s3' : u'baz' })
        self.assertEqual(response.getStatus(), 200)
        self.assert_(updatedMsgExists(response.getBody()))

        # check new values in object
        object = traverse(self.getRootFolder(), 'test')
        self.assertEqual(object.s1, u'foo')
        self.assertEqual(object.s2, u'bar')
        self.assertEqual(object.s3, u'baz')


    def test_invalid_type(self):
        """Tests textarea widget's handling of invalid unicode input.

        The text widget will succeed in converting any form input into
        unicode.
        """
        self.getRootFolder()['test'] = TextTest()
        transaction.commit()

        # submit invalid type for text
        response = self.publish('/test/edit.html', form={
            'UPDATE_SUBMIT' : '',
            'field.s1' : 123 }) # not unicode
        self.assertEqual(response.getStatus(), 200)
        # Note: We don't have a invalid field value
        # since we convert the value to unicode
        self.assert_(not validationErrorExists(
            's1', 'Object is of wrong type.', response.getBody()))


    def test_missing_value(self):
        self.getRootFolder()['test'] = TextTest()
        transaction.commit()

        # submit missing values for s2 and s3
        response = self.publish('/test/edit.html', form={
            'UPDATE_SUBMIT' : '',
            'field.s1' : u'foo',
            'field.s2' : '',
            'field.s3' : '' })
        self.assertEqual(response.getStatus(), 200)
        self.assert_(updatedMsgExists(response.getBody()))

        # check new value in object
        object = traverse(self.getRootFolder(), 'test')
        self.assertEqual(object.s1, u'foo')
        self.assertEqual(object.s2, u'')   # default missing_value
        self.assertEqual(object.s3, None)  # None is s3's missing_value


    def test_required_validation(self):
        self.getRootFolder()['test'] = TextTest()
        transaction.commit()

        # submit missing values for required field s1
        response = self.publish('/test/edit.html', form={
            'UPDATE_SUBMIT' : '',
            'field.s1' : '',
            'field.s2' : '',
            'field.s3' : '' })
        self.assertEqual(response.getStatus(), 200)

        # confirm error msgs
        self.assert_(missingInputErrorExists('s1', response.getBody()))
        self.assert_(not missingInputErrorExists('s2', response.getBody()))
        self.assert_(not missingInputErrorExists('s3', response.getBody()))


    def test_length_validation(self):
        self.getRootFolder()['test'] = TextTest()
        transaction.commit()

        # submit value for s1 that is too short
        response = self.publish('/test/edit.html', form={
            'UPDATE_SUBMIT' : '',
            'field.s1' : u'a' })
        self.assertEqual(response.getStatus(), 200)
        self.assert_(validationErrorExists(
            's1', 'Value is too short', response.getBody()))

        # submit value for s1 that is too long
        response = self.publish('/test/edit.html', form={
            'UPDATE_SUBMIT' : '',
            'field.s1' : u'12345678901' })
        self.assertEqual(response.getStatus(), 200)
        self.assert_(validationErrorExists('s1', 'Value is too long',
            response.getBody()))


    def test_omitted_value(self):
        self.getRootFolder()['test'] = TextTest()
        transaction.commit()

        # confirm default values
        object = traverse(self.getRootFolder(), 'test')
        self.assertEqual(object.s1, '')
        self.assertEqual(object.s2, u'foo')
        self.assert_(object.s3 is None)

        # submit change with only s2 present -- note that required
        # field s1 is omitted, which should not cause a validation error
        response = self.publish('/test/edit.html', form={
            'UPDATE_SUBMIT' : '',
            'field.s2' : u'bar' })
        self.assertEqual(response.getStatus(), 200)
        self.assert_(updatedMsgExists(response.getBody()))

        # check new values in object
        object = traverse(self.getRootFolder(), 'test')
        self.assertEqual(object.s1, '')
        self.assertEqual(object.s2, u'bar')
        self.assert_(object.s3 is None)


    def test_conversion(self):
        self.getRootFolder()['test'] = TextTest()
        transaction.commit()

        # confirm that line terminators are converted correctly on post
        response = self.publish('/test/edit.html', form={
            'UPDATE_SUBMIT' : '',
            'field.s2' : u'line1\r\nline2' }) # CRLF per RFC 822
        self.assertEqual(response.getStatus(), 200)
        self.assert_(updatedMsgExists(response.getBody()))
        object = traverse(self.getRootFolder(), 'test')
        self.assertEqual(object.s2, u'line1\nline2')

        # confirm conversion to HTML
        response = self.publish('/test/edit.html')
        self.assertEqual(response.getStatus(), 200)
        self.assert_(patternExists('line1\r\nline2', response.getBody()))


def test_suite():
    suite = unittest.TestSuite()
    Test.layer = AppFormLayer
    suite.addTest(unittest.makeSuite(Test))
    return suite

if __name__=='__main__':
    unittest.main(defaultTest='test_suite')



##############################################################################
#
# Copyright (c) 2001, 2002 Zope Corporation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Test object widget

$Id: test_functional_objectwidget.py 81040 2007-10-24 15:27:12Z srichter $
"""
import unittest
from zope.testing import doctest
from zope.interface import Interface, implements
from zope.publisher.browser import TestRequest
from zope.schema import Object, TextLine
import zope.security.checker
from zope.app.form.browser import ObjectWidget
from zope.app.testing.functional import BrowserTestCase
from zope.app.form.browser.tests import support
from zope.app.form.testing import AppFormLayer

class ITestContact(Interface):
    name = TextLine()
    email = TextLine()
    
class TestContact(object):
    implements(ITestContact)

class Test(BrowserTestCase, support.VerifyResults):
    
    def setUp(self):
        BrowserTestCase.setUp(self)
        self.field = Object(ITestContact, __name__=u'foo')

    def test_new(self):
        request = TestRequest()
        widget = ObjectWidget(self.field, request, TestContact)
        self.assertEquals(int(widget.hasInput()), 0)
        check_list = (
            'input', 'name="field.foo.name"',
            'input', 'name="field.foo.email"'
        )
        self.verifyResult(widget(), check_list)

    def test_edit(self):
        request = TestRequest(form={
            'field.foo.name': u'fred',
            'field.foo.email': u'fred@fred.com'
            })
        widget = ObjectWidget(self.field, request, TestContact)
        self.assertEquals(int(widget.hasInput()), 1)
        o = widget.getInputValue()
        self.assertEquals(hasattr(o, 'name'), 1)
        self.assertEquals(o.name, u'fred')
        self.assertEquals(o.email, u'fred@fred.com')
        check_list = (
            'input', 'name="field.foo.name"', 'value="fred"',
            'input', 'name="field.foo.email"', 'value="fred@fred.com"',
        )
        self.verifyResult(widget(), check_list)

def test_suite():
    suite = unittest.TestSuite()
    Test.layer = AppFormLayer
    suite.addTest(unittest.makeSuite(Test))
    return suite

if __name__=='__main__':
    unittest.main(defaultTest='test_suite')






##############################################################################
#
# Copyright (c) 2001, 2002 Zope Corporation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Test the Choice display and edit widget (function).

$Id: test_choicewidget.py 73521 2007-03-24 23:49:37Z fdrake $
"""
import unittest
from zope.publisher.interfaces.browser import IBrowserRequest
from zope.publisher.browser import TestRequest
from zope.schema.interfaces import IChoice, IIterableVocabulary
from zope.schema import Choice

from zope.app.testing import ztapi
from zope.app.testing.placelesssetup import PlacelessSetup
from zope.app.form.interfaces import IInputWidget, IDisplayWidget
from zope.app.form.browser import ChoiceDisplayWidget, ChoiceInputWidget
from zope.app.form.browser import ItemDisplayWidget, DropdownWidget


class ChoiceWidgetTest(PlacelessSetup, unittest.TestCase):

    def test_ChoiceDisplayWidget(self):
        ztapi.provideMultiView((IChoice, IIterableVocabulary), IBrowserRequest,
                               IDisplayWidget, '', ItemDisplayWidget)
        field = Choice(values=[1, 2, 3])
        bound = field.bind(object())
        widget = ChoiceDisplayWidget(bound, TestRequest())
        self.assert_(isinstance(widget, ItemDisplayWidget))
        self.assertEqual(widget.context, bound)
        self.assertEqual(widget.vocabulary, bound.vocabulary)


    def test_ChoiceInputWidget(self):
        ztapi.provideMultiView((IChoice, IIterableVocabulary), IBrowserRequest,
                               IInputWidget, '', DropdownWidget)
        field = Choice(values=[1, 2, 3])
        bound = field.bind(object())
        widget = ChoiceInputWidget(bound, TestRequest())
        self.assert_(isinstance(widget, DropdownWidget))
        self.assertEqual(widget.context, bound)
        self.assertEqual(widget.vocabulary, bound.vocabulary)
        


def test_suite():
    return unittest.TestSuite((
        unittest.makeSuite(ChoiceWidgetTest),
        ))

if __name__=='__main__':
    unittest.main(defaultTest='test_suite')



##############################################################################
#
# Copyright (c) 2004 Zope Corporation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Select Widget Tests

$Id: test_itemswidget.py 93357 2008-11-26 09:30:59Z zagy $
"""
import unittest

from zope.interface import Interface, implements
from zope.schema import Choice, List, Set, TextLine, FrozenSet
from zope.schema.vocabulary import SimpleVocabulary, SimpleTerm
from zope.publisher.browser import TestRequest

from zope.app.form.interfaces import ConversionError
import zope.app.form.browser.itemswidgets
from zope.app.form.browser.itemswidgets import ItemsWidgetBase
from zope.app.form.browser.itemswidgets import ItemDisplayWidget
from zope.app.form.browser.itemswidgets import ItemsMultiDisplayWidget
from zope.app.form.browser.itemswidgets import ListDisplayWidget
from zope.app.form.browser.itemswidgets import SetDisplayWidget
from zope.app.form.browser.itemswidgets import ItemsEditWidgetBase
from zope.app.form.browser.itemswidgets import SelectWidget, DropdownWidget
from zope.app.form.browser.itemswidgets import RadioWidget
from zope.app.form.browser.itemswidgets import ItemsMultiEditWidgetBase
from zope.app.form.browser.itemswidgets import MultiSelectWidget
from zope.app.form.browser.itemswidgets import OrderedMultiSelectWidget
from zope.app.form.browser.itemswidgets import MultiCheckBoxWidget
from zope.app.form.browser.tests.support import VerifyResults
from zope.app.testing.placelesssetup import PlacelessSetup

vocab = SimpleVocabulary(
    [SimpleTerm(value, token, title) for value, token, title in (
        ('one', 'token1', 'One'),
        ('two', 'token2', 'Two'),
        ('three', 'token3', 'Three'))])

class ICollector(Interface):
    choice = Choice(
        title=u"Number",
        description=u"The Number",
        # we want to be able to distinguish between tokens and values
        vocabulary=vocab,
        required=True)

    numbers = List(
        title=u"Numbers",
        description=u"The Numbers",
        value_type=choice,
        required=False)

    letters = Set(
        title=u"Letters",
        description=u"The Letters",
        value_type=choice,
        required=False)

    frozenLetters = FrozenSet(
        title=u"Frozen Letters",
        description=u"The Frozen Letters",
        value_type=choice,
        required=False)


class Collector(object):
    implements(ICollector)

    def __init__(self, numbers=None):
        self.numbers = numbers or []


class ItemsWidgetBaseTest(VerifyResults, PlacelessSetup, unittest.TestCase):

    _widget = ItemsWidgetBase
    _field = ICollector.get('choice')
    _vocabulary = _field.vocabulary

    def _makeWidget(self, form=None, nums=None):
        request = TestRequest(form=form or {})
        collector = Collector(nums)
        bound = self._field.bind(collector)
        return self._widget(bound, self._vocabulary, request)

    def test_setPrefix(self):
        widget = self._makeWidget()
        name = self._field.getName()
        # Default prefix
        self.assertEqual(widget._prefix, 'field.')
        self.assertEqual(widget.name, 'field.%s' %name)
        self.assertEqual(widget.empty_marker_name,
                         'field.%s-empty-marker' %name)
        # Declaring custom prefix
        widget.setPrefix('foo')
        self.assertEqual(widget._prefix, 'foo.')
        self.assertEqual(widget.name, 'foo.%s' %name)
        self.assertEqual(widget.empty_marker_name,
                         'foo.%s-empty-marker' %name)
        # Declaring empty prefix
        widget.setPrefix('')
        self.assertEqual(widget._prefix, '')
        self.assertEqual(widget.name, name)
        self.assertEqual(widget.empty_marker_name,
                         '%s-empty-marker' %name)

    def test_convertTokensToValues(self):
        widget = self._makeWidget()
        self.assertEqual(widget.convertTokensToValues(['token1', 'token2']),
                         ['one', 'two'])


class ItemDisplayWidgetTest(ItemsWidgetBaseTest):

    _widget = ItemDisplayWidget

    def test_setVocabulary(self):
        widget = self._makeWidget()
        self.assert_(widget.vocabulary is not None)
        self.assertEqual(widget.vocabulary, self._field.vocabulary)

    def test__call__(self):
        widget = self._makeWidget()
        self.assertEqual(widget(), '')
        widget = self._makeWidget(form={'field.choice': 'token1'})
        self.assertEqual(widget(), 'One')

    def test_not_required(self):
        self.failIf(self._makeWidget().required)


class ItemsMultiDisplayWidgetTest(ItemsWidgetBaseTest):

    _widget = ItemsMultiDisplayWidget
    _field = ICollector.get('numbers')
    _vocabulary = _field.value_type.vocabulary
    _tag = 'ol'

    def test__call__(self):
        widget = self._makeWidget()
        self.assertEqual(widget(), '')
        widget = self._makeWidget(form={'field.numbers': ['token1', 'token2']})
        self.assertEqual(
            widget(),
            '<%s id="field.numbers" >'
            '<li>One</li>\n<li>Two</li>'
            '</%s>' %(self._tag, self._tag))

    def test_renderItems(self):
        widget = self._makeWidget()
        self.assertEqual(
            widget.renderItems(['one', 'two']),
            [u'<li>One</li>', u'<li>Two</li>'])
        self.assertRaises(LookupError, widget.renderItems, 'one')
        self.assertRaises(TypeError, widget.renderItems, 1)


    def test_not_required(self):
        numbers = List(value_type=ICollector['choice']).bind(Collector(None))
        request = TestRequest()
        widget = self._widget(numbers, self._vocabulary, request)
        self.failIf(widget.required)


class ListDisplayWidgetTest(ItemsMultiDisplayWidgetTest):
    _widget = ListDisplayWidget
    _tag = 'ol'


class SetDisplayWidgetTest(ItemsMultiDisplayWidgetTest):
    _widget = SetDisplayWidget
    _tag = 'ul'


class ItemsEditWidgetBaseTest(ItemsWidgetBaseTest):

    _widget = ItemsEditWidgetBase

    def test_div(self):
        widget = self._makeWidget()
        self.assertEqual(widget._div('', ''), '')
        self.assertEqual(widget._div('foo', ''), '')
        self.assertEqual(widget._div('', 'bar'), '<div>\nbar\n</div>')
        self.assertEqual(widget._div('foo', 'bar'),
                         '<div class="foo">\nbar\n</div>')
        self.assertEqual(widget._div('foo', 'bar', style='blah'),
                         '<div class="foo" style="blah">\nbar\n</div>')

    def test_renderItem(self):
        widget = self._makeWidget()
        self.assertEqual(widget.renderItem('', 'Foo', 'foo', '', None),
                         '<option value="foo">Foo</option>')
        self.assertEqual(widget.renderItem('', 'Foo', 'foo', '', 'klass'),
                         '<option class="klass" value="foo">Foo</option>')

    def test_renderSelectedItem(self):
        widget = self._makeWidget()
        self.verifyResult(
          widget.renderSelectedItem('', 'Foo', 'foo', '', None),
          ['<option', 'value="foo"', 'selected="selected"', '>Foo</option>'])
        self.verifyResult(
          widget.renderSelectedItem('', 'Foo', 'foo', '', 'klass'),
          ['<option', 'class="klass"', 'value="foo"', 'selected="selected"',
           '>Foo</option>'])

    def test_renderItemsWithValues(self):
        widget = self._makeWidget()
        self.assertEqual(
            widget.renderItemsWithValues(['one', 'two']),
            [u'<option selected="selected" value="token1">One</option>',
             u'<option selected="selected" value="token2">Two</option>',
             u'<option value="token3">Three</option>'])
        self.assertEqual(
            widget.renderItemsWithValues([]),
            [u'<option value="token1">One</option>',
             u'<option value="token2">Two</option>',
             u'<option value="token3">Three</option>'])

# This test is disabled because it tests for the presense of a missfeature,
# which has been removed.  Did someone actually *want* this?
##     def test_error(self):
##         widget = self._makeWidget(form={'field.choice': 'ten'})
##         widget.setPrefix('field.')
##         widget._getFormValue()
##         self.assert_(isinstance(widget._error, ConversionError))

    def test_hidden(self):
        widget = self._makeWidget(form={'field.choice': 'token2'})
        widget.setPrefix('field.')
        widget.context.required = False
        self.verifyResult(
            widget.hidden(),
            ['<input', 'type="hidden"', 'value="token2"', 'id="field.choice"',
             'name="field.choice"'])

class SelectWidgetTest(ItemsEditWidgetBaseTest):

    _widget = SelectWidget
    _size = 5

    def test__call__(self):
        widget = self._makeWidget(form={'field.choice': 'token1'})
        widget.setPrefix('field.')
        widget.context.required = False
        self.assertEqual(
            widget(),
            '<div>\n'
            '<div class="value">\n'
            '<select id="field.choice" name="field.choice" size="%i" >\n'
            '<option value="">(no value)</option>\n'
            '<option selected="selected" value="token1">One</option>\n'
            '<option value="token2">Two</option>\n'
            '<option value="token3">Three</option>\n'
            '</select>\n</div>\n'
            '<input name="field.choice-empty-marker" '
            'type="hidden" value="1" />\n</div>' %self._size)

    def test_renderValue(self):
        widget = self._makeWidget()
        widget.setPrefix('field.')
        self.assertEqual(
            widget.renderValue('one'),
            '<select id="field.choice" name="field.choice" size="%i" >\n'
            '<option selected="selected" value="token1">One</option>\n'
            '<option value="token2">Two</option>\n'
            '<option value="token3">Three</option>\n'
            '</select>' %self._size)

    def test_renderItems(self):
        widget = self._makeWidget()
        widget.setPrefix('field.')
        self.assertEqual(
            widget.renderItems('one'),
            [u'<option selected="selected" value="token1">One</option>',
             u'<option value="token2">Two</option>',
             u'<option value="token3">Three</option>'])
        self.assertEqual(
            widget.renderItems('two'),
            [u'<option value="token1">One</option>',
             u'<option selected="selected" value="token2">Two</option>',
             u'<option value="token3">Three</option>'])
        self.assertEqual(
            widget.renderItems(None),
            [u'<option value="token1">One</option>',
             u'<option value="token2">Two</option>',
             u'<option value="token3">Three</option>'])

    def test_renderItems_notRequired(self):
        widget = self._makeWidget()
        widget.setPrefix('field.')
        widget.context.required = False
        self.assertEqual(
            widget.renderItems([]),
            [u'<option value="">(no value)</option>',
             u'<option value="token1">One</option>',
             u'<option value="token2">Two</option>',
             u'<option value="token3">Three</option>'])

    def test_renderItems_firstItem(self):
        widget = self._makeWidget()
        widget.setPrefix('field.')
        widget.firstItem = True
        self.assertEqual(
            widget.renderItems(widget._toFormValue(widget.context.missing_value)),
            [u'<option value="token1">One</option>',
             u'<option value="token2">Two</option>',
             u'<option value="token3">Three</option>'])


class DropdownWidgetTest(SelectWidgetTest):

    _widget = DropdownWidget
    _size = 1

    def test_renderItems_firstItem(self):
        widget = self._makeWidget()
        widget.setPrefix('field.')
        widget.firstItem = True
        self.assertEqual(
            widget.renderItems(widget._toFormValue(widget.context.missing_value)),
            [u'<option selected="selected" value="">(no value)</option>',
             u'<option value="token1">One</option>',
             u'<option value="token2">Two</option>',
             u'<option value="token3">Three</option>'])
        try:
            # test BBB starting with 3.6.0
            zope.app.form.browser.itemswidgets.EXPLICIT_EMPTY_SELECTION = False
            self.assertEqual(
                widget.renderItems(widget._toFormValue(widget.context.missing_value)),
                [u'<option value="token1">One</option>',
                 u'<option value="token2">Two</option>',
                 u'<option value="token3">Three</option>'])
        finally:
            zope.app.form.browser.itemswidgets.EXPLICIT_EMPTY_SELECTION = True


class RadioWidgetTest(ItemsEditWidgetBaseTest):

    _widget = RadioWidget

    def test_renderItem(self):
        widget = self._makeWidget()
        self.verifyResult(
            widget.renderItem('', 'Foo', 'foo', 'bar', None),
            ['<label', '<input', 'type="radio"', 'name="bar"', 'value="foo"',
             'class="radioType"', '>&nbsp;Foo'])
        self.verifyResult(
            widget.renderItem('bar', 'Foo', 'foo', 'bar', 'klass'),
            ['<input', 'type="radio"', 'name="bar"', 'value="foo"',
             'class="klass radioType"', '>&nbsp;Foo'])

    def test_renderSelectedItem(self):
        widget = self._makeWidget()
        self.verifyResult(
            widget.renderSelectedItem('', 'Foo', 'foo', 'bar', 'klass'),
            ['<label', '<input', 'type="radio"', 'name="bar"', 'value="foo"',
             'checked="checked"', '>&nbsp;Foo'])
        self.verifyResult(
            widget.renderSelectedItem('', 'Foo', 'foo', 'bar', 'klass'),
            ['<label', '<input', 'type="radio"', 'name="bar"', 'value="foo"',
             'class="klass radioType"', 'checked="checked"', '>&nbsp;Foo'])

    def test_renderItemsWithValues(self):
        widget = self._makeWidget()
        items = widget.renderItemsWithValues(['one'])
        values = [('token1','One'), ('token2','Two'), ('token3','Three')]
        for index, item in enumerate(items):
            self.verifyResult(
                item,
                ['<label', '<input', 'class="radioType"', 'name="field.choice"',
                 'id="field.choice.%i' %index, 'type="radio"',
                 'value="%s"' %values[index][0],
                 '&nbsp;%s' %values[index][1]])
        self.verifyResult(items[0], ['checked="checked"'])

    def test_renderItems(self):
        widget = self._makeWidget()
        items = widget.renderItems('one')
        values = [('token1','One'), ('token2','Two'), ('token3','Three')]
        for index, item in enumerate(items):
            self.verifyResult(
                item,
                ['<label', '<input', 'class="radioType"', 'name="field.choice"',
                 'id="field.choice.%i' %index, 'type="radio"',
                 'value="%s"' %values[index][0], '&nbsp;%s' %values[index][1]])
        self.verifyResult(items[0], ['checked="checked"'])

    def test_renderItems_notRequired(self):
        widget = self._makeWidget()
        widget.context.required = False
        items = widget.renderItems([])
        values = [('', '(no value)'),
                  ('token1','One'),
                  ('token2','Two'),
                  ('token3','Three')]
        for index, item in enumerate(items):
            self.verifyResult(
                item,
                ['<label', '<input', 'class="radioType"',
                 'name="field.choice"', 'type="radio"',
                 'value="%s"' %values[index][0], '&nbsp;%s' %values[index][1]])

    def test_renderItems_firstItem(self):
        widget = self._makeWidget()
        items = widget.renderItems(
                widget._toFormValue(widget.context.missing_value))
        values = [('token1','One'), ('token2','Two'), ('token3','Three')]
        for index, item in enumerate(items):
            self.verifyResult(
                item,
                ['<label', '<input', 'class="radioType"',
                 'name="field.choice"', 'id="field.choice.%i"' % index,
                 'type="radio"', 'value="%s"' % values[index][0],
                 '&nbsp;%s' % values[index][1]])

    def test_renderValue(self):
        widget = self._makeWidget()
        self.verifyResult(widget.renderValue(None), ['<br /><label for='])
        widget.orientation = 'horizontal'
        self.verifyResult(widget.renderValue(None),
                          ['&nbsp;&nbsp;<label for='])


class ItemsMultiEditWidgetBaseTest(ItemsEditWidgetBaseTest):

    _widget = ItemsMultiEditWidgetBase
    _field = ICollector.get('numbers')
    _vocabulary = _field.value_type.vocabulary

    def test_renderValue(self):
        widget = self._makeWidget()
        self.verifyResult(
            widget.renderValue(['one', 'two']),
            ['<select', 'multiple="multiple"', 'name="field.numbers:list"',
             'size="5"', '><option', 'selected="selected"', 'value="token1"',
             '>One</option>\n', 'value="token2"', '>Two</option>\n',
             'value="token3"', '>Three</option>', '</select>'])

    def test_renderItemsWithValues(self):
        widget = self._makeWidget()
        self.assertEqual(
            widget.renderItemsWithValues(['one', 'two']),
            [u'<option selected="selected" value="token1">One</option>',
             u'<option selected="selected" value="token2">Two</option>',
             u'<option value="token3">Three</option>'])
        self.assertEqual(
            widget.renderItemsWithValues([]),
            [u'<option value="token1">One</option>',
             u'<option value="token2">Two</option>',
             u'<option value="token3">Three</option>'])

# This test is disabled because it tests for the presense of a missfeature,
# which has been removed.  Did someone actually *want* this?
##     def test_error(self):
##         widget = self._makeWidget(form={'field.numbers': ['ten']})
##         widget.setPrefix('field.')
##         widget._getFormValue()
##         self.assert_(isinstance(widget._error, ConversionError))

    def test_hidden(self):
        widget = self._makeWidget(
            form={'field.numbers': ['two','three']})
        widget.setPrefix('field.')
        widget.context.required = False
        self.verifyResult(
            widget.hidden(),
            ['<input', 'type="hidden"', 'value="token2"', 'id="field.numbers"',
             'name="field.numbers:list"', 'value="token3"'])

    def test_getInputValue(self):
        widget = self._makeWidget(form={'field.numbers': ['token2', 'token3']})
        widget.setPrefix('field.')
        self.assertEqual(widget.getInputValue(), ['two', 'three'])

        self._field = ICollector.get('letters')
        widget = self._makeWidget(form={'field.letters-empty-marker': '1'})
        widget.setPrefix('field.')
        self.assertEqual(widget.getInputValue(), set([]))
        widget = self._makeWidget(form={'field.letters': ['token2', 'token3']})
        widget.setPrefix('field.')
        self.assertEqual(widget.getInputValue(), set(['two', 'three']))

        self._field = ICollector.get('frozenLetters')
        widget = self._makeWidget(form={'field.frozenLetters-empty-marker':
                                        '1'})
        widget.setPrefix('field.')
        field_value = widget.getInputValue()
        self.assertEqual(field_value, frozenset())
        widget = self._makeWidget(form={'field.frozenLetters':
                                        ['token2', 'token3']})
        widget.setPrefix('field.')
        field_value = widget.getInputValue()
        self.assertEqual(field_value, frozenset(['two', 'three']))
        self.assert_(isinstance(field_value, frozenset))


class MultiSelectWidgetTest(ItemsMultiEditWidgetBaseTest):

    _widget = MultiSelectWidget


class OrderedMultiSelectWidgetTest(ItemsMultiEditWidgetBaseTest):

    _widget = OrderedMultiSelectWidget

    def test_choices(self):
        widget = self._makeWidget()
        choices = [choice['text'] for choice in widget.choices()]
        choices.sort()
        self.assertEqual(choices, ['One', 'Three', 'Two'])

    def test_selected(self):
        widget = self._makeWidget(nums=['one'])
        widget._data = ['two']
        selected = [select['text'] for select in widget.selected()]
        selected.sort()
        self.assertEqual(selected, ['One', 'Two'])
        widget._data = ['one']
        selected = [select['text'] for select in widget.selected()]
        selected.sort()
        self.assertEqual(selected, ['One'])


class MultiCheckBoxWidgetTest(ItemsMultiEditWidgetBaseTest):

    _widget = MultiCheckBoxWidget

    def test_renderItem(self):
        widget = self._makeWidget()
        self.verifyResult(
            widget.renderItem('', 'Foo', 'foo', 'bar', None),
            ['<input', 'type="checkbox"', 'name="bar"', 'value="foo"',
             'class="checkboxType"', '>&nbsp;Foo'])
        self.verifyResult(
            widget.renderItem('bar', 'Foo', 'foo', 'bar', 'klass'),
            ['<input', 'type="checkbox"', 'name="bar"', 'value="foo"',
             'class="klass checkboxType"', '>&nbsp;Foo'])

    def test_renderSelectedItem(self):
        widget = self._makeWidget()
        self.verifyResult(
            widget.renderSelectedItem('', 'Foo', 'foo', 'bar', 'klass'),
            ['<input', 'type="checkbox"', 'name="bar"', 'value="foo"',
             'checked="checked"', '>&nbsp;Foo'])
        self.verifyResult(
            widget.renderSelectedItem('', 'Foo', 'foo', 'bar', 'klass'),
            ['<input', 'type="checkbox"', 'name="bar"', 'value="foo"',
             'class="klass checkboxType"', 'checked="checked"', '>&nbsp;Foo'])

    def test_renderValue(self):
        widget = self._makeWidget()
        self.verifyResult(widget.renderValue(None), ['<br /><label for='])
        widget.orientation='horizontal'
        self.verifyResult(widget.renderValue(None),
                          ['&nbsp;&nbsp;<label for='])

    def test_renderItemsWithValues(self):
        widget = self._makeWidget()
        items = widget.renderItemsWithValues(['one'])
        values = [('token1','One'), ('token2','Two'), ('token3','Three')]
        for index, item in enumerate(items):
            self.verifyResult(
                item,
                ['<input', 'class="checkboxType',
                 'id="field.numbers.%i"' %index, 'type="checkbox"',
                 'value="%s"' % values[index][0],
                 '/>&nbsp;%s' % values[index][1]])

        self.verifyResult(items[0], ['checked="checked"'])


def test_suite():
    suite = unittest.makeSuite(ItemDisplayWidgetTest)
    suite.addTest(unittest.makeSuite(ItemsMultiDisplayWidgetTest))
    suite.addTest(unittest.makeSuite(ListDisplayWidgetTest))
    suite.addTest(unittest.makeSuite(SetDisplayWidgetTest))
    suite.addTest(unittest.makeSuite(ItemsEditWidgetBaseTest))
    suite.addTest(unittest.makeSuite(SelectWidgetTest))
    suite.addTest(unittest.makeSuite(DropdownWidgetTest))
    suite.addTest(unittest.makeSuite(RadioWidgetTest))
    suite.addTest(unittest.makeSuite(ItemsMultiEditWidgetBaseTest))
    suite.addTest(unittest.makeSuite(MultiSelectWidgetTest))
    suite.addTest(unittest.makeSuite(OrderedMultiSelectWidgetTest))
    suite.addTest(unittest.makeSuite(MultiCheckBoxWidgetTest))
    return suite

if __name__ == '__main__':
    unittest.main(defaultTest="test_suite")



##############################################################################
#
# Copyright (c) 2004 Zope Corporation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Tests for the <widget> subdirective for the generated form pages.

$Id: test_widgetdirective.py 73522 2007-03-24 23:51:08Z fdrake $
"""
import unittest

import zope.component
import zope.interface
import zope.configuration.xmlconfig
import zope.publisher.browser
import zope.schema

import zope.app.container.interfaces
import zope.app.form.browser.interfaces
import zope.app.form.interfaces
from zope.app.form.tests import utils
import zope.app.testing.placelesssetup

__docformat__ = "reStructuredText"


class IContent(zope.interface.Interface):

    field = zope.schema.TextLine(
        title=u"Field",
        description=u"Sample input field",
        required=False,
        )


class Content(object):

    zope.interface.implements(IContent)
    __Security_checker__ = utils.SchemaChecker(IContent)

    __parent__ = None
    __name__ = "sample-content"

    field = None


class Adding(object):

    zope.interface.implements(
        zope.app.container.interfaces.IAdding)

    def add(self, content):
        self.content = content


class WidgetDirectiveTestCase(zope.app.testing.placelesssetup.PlacelessSetup,
                              unittest.TestCase):

    def setUp(self):
        super(WidgetDirectiveTestCase, self).setUp()
        zope.configuration.xmlconfig.file("widgetDirectives.zcml",
                                          zope.app.form.browser.tests)

    def get_widget(self, name, context):
        request = zope.publisher.browser.TestRequest()
        view = zope.component.getMultiAdapter((context, request), name=name)
        return view.field_widget

    def test_addform_widget_without_class(self):
        w = self.get_widget("add.html", Adding())
        self.assert_(zope.app.form.interfaces.IInputWidget.providedBy(w))
        self.assertEqual(w.extraAttr, "42")

    def test_editform_widget_without_class(self):
        w = self.get_widget("edit.html", Content())
        self.assert_(zope.app.form.interfaces.IInputWidget.providedBy(w))
        self.assertEqual(w.extraAttr, "84")

    def test_subeditform_widget_without_class(self):
        w = self.get_widget("subedit.html", Content())
        self.assert_(zope.app.form.interfaces.IInputWidget.providedBy(w))
        self.assertEqual(w.extraAttr, "168")


def test_suite():
    return unittest.makeSuite(WidgetDirectiveTestCase)



##############################################################################
#
# Copyright (c) 2001, 2002 Zope Corporation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""RadioWidget Tests

$Id: test_functional_selectwidget.py 81040 2007-10-24 15:27:12Z srichter $
"""
import unittest
import transaction
from persistent import Persistent

import zope.security.checker
from zope.interface import Interface, implements
from zope.schema import TextLine, Choice
from zope.traversing.api import traverse

from zope.app.form.testing import AppFormLayer
from zope.app.form.browser.tests.support import *
from zope.app.testing.functional import BrowserTestCase

class IRadioTest(Interface):

    s3 = Choice(
        required=False,
        values=(u'Bob', u'is', u'Your', u'Uncle'))

    s4 = Choice(
        required=True,
        values=(u'1', u'2', u'3'))

class RadioTest(Persistent):

    implements(IRadioTest)

    def __init__(self):
        self.s3 = None
        self.s4 = u'1'

class Test(BrowserTestCase):

    def setUp(self):
        BrowserTestCase.setUp(self)
        registerEditForm(IRadioTest)
        defineSecurity(RadioTest, IRadioTest)

    def test_display_editform(self):
        self.getRootFolder()['test'] = RadioTest()
        transaction.commit()

        test = self.getRootFolder()['test']
        test.s3 = u"Bob"

        # display edit view
        response = self.publish('/test/edit.html')
        self.assertEqual(response.getStatus(), 200)

        # S3
        self.assert_(patternExists(
            '<select .* name="field.s3".*>',
            response.getBody()))
        self.assert_(patternExists(
            '<option selected="selected" value="">',
            response.getBody()))
        self.assert_(patternExists(
            '<option value="Bob">',
            response.getBody()))
        self.assert_(patternExists(
            '<option value="is">',
            response.getBody()))
        self.assert_(patternExists(
            '<option value="Your">',
            response.getBody()))
        self.assert_(patternExists(
            '<option value="Uncle">',
            response.getBody()))

        # S4
        joined_body = "".join(response.getBody().split("\n"))
        self.failIf(patternExists(
            '<select.*name="field.s4".*>.*<option.*value="".*>',
            joined_body))
        self.assert_(patternExists(
            '<select .* name="field.s4".*>',
            response.getBody()))
        self.assert_(patternExists(
            '<option selected="selected" value="1">',
            response.getBody()))
        self.assert_(patternExists(
            '<option value="2">',
            response.getBody()))
        self.assert_(patternExists(
            '<option value="3">',
            response.getBody()))

        response = self.publish('/test/edit.html', form={
            'UPDATE_SUBMIT': '',
            'field.s3': u'Bob',
            'field.s4': u'2'})
        self.assert_(patternExists(
            '<option selected="selected" value="Bob">',
            response.getBody()))
        self.assert_(patternExists(
            '<option selected="selected" value="2">',
            response.getBody()))

        response = self.publish('/test/edit.html')
        self.assert_(patternExists(
            '<option selected="selected" value="Bob">',
            response.getBody()))
        self.assert_(patternExists(
            '<option selected="selected" value="2">',
            response.getBody()))

        response = self.publish('/test/edit.html', form={
            'UPDATE_SUBMIT': '',
            'field.s3': u''})
        self.assert_(patternExists(
            '<option selected="selected" value="">',
            response.getBody()))
        self.assert_(patternExists(
            '<option selected="selected" value="2">',
            response.getBody()))

        response = self.publish('/test/edit.html')
        self.assert_(patternExists(
            '<option selected="selected" value="">',
            response.getBody()))

def test_suite():
    suite = unittest.TestSuite()
    Test.layer = AppFormLayer
    suite.addTest(unittest.makeSuite(Test))
    return suite

if __name__=='__main__':
    unittest.main(defaultTest='test_suite')




##############################################################################
#
# Copyright (c) 2001, 2002 Zope Corporation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Password Widget Tests

$Id: test_passwordwidget.py 69218 2006-07-20 04:12:28Z baijum $
"""
import unittest
from zope.testing import doctest
from zope.app.form.interfaces import IInputWidget
from zope.app.form.browser import PasswordWidget
from zope.app.form.browser.tests.test_browserwidget import SimpleInputWidgetTest
from zope.interface.verify import verifyClass

class PasswordWidgetTest(SimpleInputWidgetTest):
    """Documents and tests the password widget.

        >>> verifyClass(IInputWidget, PasswordWidget)
        True
    """

    _WidgetFactory = PasswordWidget

    def testProperties(self):
        self.assertEqual(self._widget.tag, 'input')
        self.assertEqual(self._widget.type, 'password')
        self.assertEqual(self._widget.cssClass, '')
        self.assertEqual(self._widget.extra, '')
        self.assertEqual(self._widget.default, '')
        self.assertEqual(self._widget.displayWidth, 20)
        self.assertEqual(self._widget.displayMaxWidth, '')

    def testRender(self):
        value = 'Foo Value'
        self._widget.setRenderedValue(value)
        check_list = ('type="password"', 'id="field.foo"',
                      'name="field.foo"', 'value=""', 'size="20"')
        self.verifyResult(self._widget(), check_list)

    def testHidden(self):
        self.assertRaises(NotImplementedError, self._widget.hidden)

def test_suite():
    return unittest.TestSuite((
        unittest.makeSuite(PasswordWidgetTest),
        doctest.DocTestSuite(),
        ))

if __name__=='__main__':
    unittest.main(defaultTest='test_suite')



##############################################################################
#
# Copyright (c) 2004 Zope Corporation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Browser widgets for text-like data

$Id: objectwidget.py 80211 2007-09-27 09:43:07Z ctheune $
"""
__docformat__ = 'restructuredtext'

from zope import component
from zope.interface import implements
from zope.schema import getFieldNamesInOrder

from zope.app.form.interfaces import IInputWidget
from zope.app.form import InputWidget
from zope.app.form.browser.widget import BrowserWidget
from zope.app.form.utility import setUpWidgets, applyWidgetsChanges
from zope.app.pagetemplate.viewpagetemplatefile import ViewPageTemplateFile
from zope.app.form.browser.interfaces import IWidgetInputErrorView


class ObjectWidgetView:

    template = ViewPageTemplateFile('objectwidget.pt')

    def __init__(self, context, request):
        self.context = context
        self.request = request

    def __call__(self):
        return self.template()


class ObjectWidget(BrowserWidget, InputWidget):
    """A widget over an Interface that contains Fields.

    ``factory``

      factory used to create content that this widget (field) represents

    ``*_widget``

      Optional CustomWidgets used to generate widgets for the fields in this
      widget
    """

    implements(IInputWidget)

    _object = None      # the object value (from setRenderedValue & request)
    _request_parsed = False

    def __init__(self, context, request, factory, **kw):
        super(ObjectWidget, self).__init__(context, request)

        # define view that renders the widget
        self.view = ObjectWidgetView(self, request)

        # factory used to create content that this widget (field)
        # represents
        self.factory = factory

        # handle foo_widget specs being passed in
        self.names = getFieldNamesInOrder(self.context.schema)
        for k, v in kw.items():
            if k.endswith('_widget'):
                setattr(self, k, v)

        # set up my subwidgets
        self._setUpEditWidgets()

    def setPrefix(self, prefix):
        super(ObjectWidget, self).setPrefix(prefix)
        self._setUpEditWidgets()

    def _setUpEditWidgets(self):
        # subwidgets need a new name
        setUpWidgets(self, self.context.schema, IInputWidget,
                         prefix=self.name, names=self.names,
                         context=self.context)

    def __call__(self):
        return self.view()

    def legendTitle(self):
        return self.context.title or self.context.__name__

    def getSubWidget(self, name):
        return getattr(self, '%s_widget' % name)

    def subwidgets(self):
        return [self.getSubWidget(name) for name in self.names]

    def hidden(self):
        """Render the object as hidden fields."""
        result = []
        for name in self.names:
            result.append(getSubwidget(name).hidden())
        return "".join(result)

    def error(self):
        if self._error:
            errormessages = []
            keys = self._error.keys(); keys.sort()
            for key in keys:
                errormessages.append(str(key) + ': ')
                errormessages.append(component.getMultiAdapter(
                    (self._error[key], self.request),
                    IWidgetInputErrorView).snippet())
                errormessages.append(str(key) + ', ')
            return ''.join(errormessages[0:-1])
        return ""

    def getInputValue(self):
        """Return converted and validated widget data.

        The value for this field will be represented as an `ObjectStorage`
        instance which holds the subfield values as attributes. It will
        need to be converted by higher-level code into some more useful
        object (note that the default EditView calls `applyChanges`, which
        does this).
        """

        errors = []
        content = self.factory()
        for name in self.names:
            try:
                setattr(content, name, self.getSubWidget(name).getInputValue())
            except Exception, e:
                errors.append(e)
                if self._error is None:
                    self._error = {}
                
                if name not in self._error:
                    self._error[name] = e

        if errors:
            raise errors[0]

        return content


    def applyChanges(self, content):
        field = self.context

        # create our new object value
        value = field.query(content, None)
        if value is None:
            # TODO: ObjectCreatedEvent here would be nice
            value = self.factory()

        # apply sub changes, see if there *are* any changes
        # TODO: ObjectModifiedEvent here would be nice
        changes = applyWidgetsChanges(self, field.schema, target=value,
                                      names=self.names)

        # if there's changes, then store the new value on the content
        if changes:
            field.set(content, value)
        # TODO: If value implements ILocation, set name to field name and
        # parent to content

        return changes

    def hasInput(self):
        """Is there input data for the field

        Return ``True`` if there is data and ``False`` otherwise.
        """
        for name in self.names:
            if self.getSubWidget(name).hasInput():
                return True
        return False

    def setRenderedValue(self, value):
        """Set the default data for the widget.

        The given value should be used even if the user has entered
        data.
        """
        # re-call setupwidgets with the content
        self._setUpEditWidgets()
        for name in self.names:
            self.getSubWidget(name).setRenderedValue(getattr(value, name, None))



##############################################################################
#
# Copyright (c) 2001, 2002 Zope Corporation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Form Utilities Tests

$Id: test_utility.py 97648 2009-03-08 08:24:24Z nadako $
"""

from zope.testing import doctest
import zope.security.checker
from zope.interface import Interface, implements
from zope.component.interfaces import ComponentLookupError
from zope.publisher.browser import TestRequest, BrowserView
from zope.security.interfaces import ForbiddenAttribute, Unauthorized
from zope.schema import Field, Int, accessors
from zope.schema.interfaces import IField, IInt

from zope.app.testing import ztapi, placelesssetup
from zope.app.form import Widget
from zope.app.form.interfaces import IWidget, IInputWidget, IDisplayWidget
from zope.app.form.interfaces import ConversionError, InputErrors, WidgetsError
from zope.app.form.interfaces import IWidgetFactory
from zope.app.form.utility import no_value, setUpWidget, setUpWidgets
from zope.app.form.utility import setUpEditWidgets, setUpDisplayWidgets
from zope.app.form.utility import getWidgetsData, viewHasInput
from zope.app.form.utility import applyWidgetsChanges
from zope.app.form.tests import utils

request = TestRequest()

class IFoo(IField):
    pass
    
class Foo(Field):
    implements(IFoo)
    
class IBar(IField):
    pass
    
class Bar(Field):
    implements(IBar)

class IBaz(IInt):
    pass

class Baz(Int):
    implements(IBaz)
    
class IContent(Interface):
    foo = Foo()
    bar = Bar()

    
class Content(object):
    implements(IContent)
    __Security_checker__ = utils.SchemaChecker(IContent)
    foo = 'Foo'

class IFooWidget(IWidget):
    pass

class IBarWidget(IWidget):
    pass

class FooWidget(Widget):
    implements(IFooWidget)
    def getPrefix(self): return self._prefix  # exposes _prefix for testing
    def getRenderedValue(self): return self._data # exposes _data for testing
    def renderedValueSet(self): return self._renderedValueSet() # for testing
    
class BarWidget(Widget):
    implements(IBarWidget)
    def getRenderedValue(self): return self._data # exposes _data for testing
    def renderedValueSet(self): return self._renderedValueSet() # for testing

class BazWidget(Widget):
    def getRenderedValue(self): return self._data # exposes _data for testing
    def renderedValueSet(self): return self._renderedValueSet() # for testing

class IExtendedContent(IContent):
    getBaz, setBaz = accessors(Baz())
    getAnotherBaz, setAnotherBaz = accessors(Baz())
    shazam = Foo()

class ExtendedContent(Content):
    implements(IExtendedContent)
    _baz = _anotherbaz = shazam = None
    def getBaz(self): return self._baz
    def setBaz(self, value): self._baz = value
    def getAnotherBaz(self): return self._anotherbaz
    def setAnotherBaz(self, value): self._anotherbaz = value

extended_checker = utils.DummyChecker(
        {'foo':True, 'bar': True, 'getBaz': True, 'setBaz': True, 
         'getAnotherBaz': True, 'setAnotherBaz': False, 'shazam': False},
        {'foo':True, 'bar': False, 'shazam': True})

def setUp():
    """Setup for tests."""
    placelesssetup.setUp()
    ztapi.browserView(IFoo, '', FooWidget, providing=IFooWidget)
    ztapi.browserView(IBar, '', BarWidget, providing=IBarWidget)
    
def tearDown():
    placelesssetup.tearDown()
    
def assertRaises(exceptionType, callable, *args):
    try:
        callable(*args)
        return False
    except Exception, e:
        return isinstance(e, exceptionType)
       
class TestSetUpWidget(object):
    
    def test_typical(self):
        """Documents and tests the typical uses of setUpWidget.
        
        >>> setUp()
          
        setUpWidget ensures that the appropriate widget exists as an
        attribute of a view. There are four required arguments to the
        function:
        
            >>> view = BrowserView(Content(), request)
            >>> name = 'foo'
            >>> field = IContent['foo']
            >>> typeView = IFooWidget
            
        setUpWidget will add the appropriate widget as attribute to view
        named 'foo_widget'.
        
            >>> hasattr(view, 'foo_widget')
            False
            >>> setUpWidget(view, name, field, typeView)
            >>> hasattr(view, 'foo_widget')
            True
            >>> IFooWidget.providedBy(view.foo_widget)
            True
            
        If the view already has an attribute with that name, it attempts to
        use the existing value to create a widget. Two types are supported:
           
            - IWidgetFactory
            - IWidget
           
        If the existing attribute value implements IWidgetFactory, it is used
        to create a widget:
           
            >>> widget = FooWidget(IContent['foo'], request)
            >>> class Factory(object):
            ...     implements(IWidgetFactory)
            ...     def __call__(self, request, context):
            ...         return widget
            >>> setattr(view, 'foo_widget', Factory())
            >>> view.foo_widget is widget
            False
            >>> setUpWidget(view, name, field, typeView)
            >>> view.foo_widget is widget
            True
            
        If the existing attribute value implements IWidget, it is used without
        modification:
           
            >>> setattr(view, 'foo_widget', widget)
            >>> IWidget.providedBy(widget)
            True
            >>> setUpWidget(view, name, field, typeView)
            >>> view.foo_widget is widget
            True

        We now have to cleanup, so that these tests can be run in a loop. We
        modified the 'IContent' interface saying that 'foo' is not mandatory,
        so we have to change that back.

            >>> IContent['foo'].required = True

        >>> tearDown()
        """
        
    def test_validation(self):
        """Documents and tests validation performed by setUpWidget.
        
        >>> setUp()
            
        setUpWidget ensures that the the view has an attribute that implements
        IWidget. If setUpWidget cannot configure a widget, it raises a
        TypeError. 
        
        E.g., if a view already has a widget attribute of the name 
        <field_name> + '_widget' that does not implement IWidgetFactory or
        IWidget, setUpWidget raises a TypeError: 
        
            >>> view = BrowserView(Content(), request)
            >>> setattr(view, 'foo_widget', 'not a widget')
            >>> assertRaises(TypeError, setUpWidget,
            ...              view, 'foo', IContent['foo'], IFooWidget)
            True
            
        Similarly, if a view has a widget attribute that implements 
        IWidgetFactory, the object created by the factory must implement IWidget.
        
            >>> class Factory(object):
            ...     implements(IWidgetFactory)
            ...     def __call__(self, request, context):
            ...         return 'not a widget'
            >>> setattr(view, 'foo_widget', Factory())
            >>> assertRaises(TypeError, setUpWidget,
            ...              view, 'foo', IContent['foo'], IFooWidget)
            True
            
        >>> tearDown()
        """
        
    def test_context(self):
        """Documents and tests the role of context in setUpWidget.
        
        >>> setUp()
        
        setUpWidget configures a widget by associating it to a bound field,
        which is a copy of a schema field that is bound to an object. The
        object the field is bound to can be explicitly specified in the
        setUpWidget 'context' argument.
        
        By default, the context used by setUpWidget is the view context:
            
            >>> context = Content()
            >>> view = BrowserView(context, request)
            >>> setUpWidget(view, 'foo', IContent['foo'], IFooWidget)
            >>> view.foo_widget.context.context is context
            True
            
        Alternatively, you can specify a context explicitly:
            
            >>> view = BrowserView(context, request)
            >>> altContext = Content()
            >>> setUpWidget(view, 'foo', IContent['foo'], IFooWidget,
            ...             context=altContext)
            >>> view.foo_widget.context.context is context
            False
            >>> view.foo_widget.context.context is altContext
            True
                    
        >>> tearDown()
        """
        
    def test_widgetLookup(self):
        """Documents and tests how widgets are looked up by type.
        
        >>> setUp()
        
        If the view does not already have a widget configured for the
        specified field name, setUpWidget will look up a widget using
        an interface specified for the widgetType argument.
        
        Widgets are typically looked up for IInputWidget and IDisplayWidget
        types. To illustrate this, we'll create two widgets, one for editing
        and another for displaying IFoo attributes. Each widget is registered
        as a view providing the appropriate widget type.
        
            >>> class EditFooWidget(Widget):
            ...     implements(IInputWidget)
            ...     def hasInput(self):
            ...         return False
            >>> ztapi.browserViewProviding(IFoo, EditFooWidget, IInputWidget)
            >>> class DisplayFooWidget(Widget):
            ...     implements(IDisplayWidget)            
            >>> ztapi.browserViewProviding(IFoo, DisplayFooWidget, 
            ...                            IDisplayWidget)
            
        A call to setUpWidget will lookup the widgets based on the specified 
        type.
            
            >>> view = BrowserView(Content(), request)
            >>> setUpWidget(view, 'foo', IContent['foo'], IInputWidget)
            >>> IInputWidget.providedBy(view.foo_widget)
            True
            >>> delattr(view, 'foo_widget')
            >>> setUpWidget(view, 'foo', IContent['foo'], IDisplayWidget)
            >>> IDisplayWidget.providedBy(view.foo_widget)
            True
            
        A ComponentError is raised if a widget is not registered for the
        specified type:
            
            >>> class IUnregisteredWidget(IWidget):
            ...     pass
            >>> delattr(view, 'foo_widget')
            >>> assertRaises(ComponentLookupError, setUpWidget,
            ...              view, 'foo', IContent['foo'], IUnregisteredWidget)
            True

        >>> tearDown()
        """
        
    def test_prefix(self):
        """Documents and tests the specification of widget prefixes.
        
        >>> setUp()
        
        Widgets support a prefix that can be used to group related widgets
        on a view. To specify the prefix for a widget, specify in the call to
        setUpWidget:
            
            >>> view = BrowserView(Content(), request)
            >>> setUpWidget(view, 'foo', IContent['foo'], IFooWidget,
            ...             prefix='mygroup')
            >>> view.foo_widget.getPrefix()
            'mygroup.'
        
        >>> tearDown()
        """
        
    def test_value(self):
        """Documents and tests values and setUpWidget.
        
        >>> setUp()
        
        setUpWidget configures the widget with the value specified in the
        'value' argument:
            
            >>> view = BrowserView(Content(), request)
            >>> setUpWidget(view, 'foo', IContent['foo'], IFooWidget, 
            ...             value='Explicit Widget Value')
            >>> view.foo_widget.renderedValueSet()
            True
            >>> view.foo_widget.getRenderedValue()
            'Explicit Widget Value'
            
        The utility module provides a marker object 'no_value' that can be
        used as setUpWidget's 'value' argument to indicate that a value 
        doesn't exist for the bound field. This may seem a bit unusual since
        None is typically used for this purpose. However, None is a valid
        value for many fields and does not indicate 'no value'.
        
        When no_value is specified in a call to setUpWidget, the effected
        widget is not configured with a value:
        
            >>> delattr(view, 'foo_widget')
            >>> setUpWidget(view, 'foo', IContent['foo'], IFooWidget,
            ...             value=no_value)
            >>> view.foo_widget.renderedValueSet()
            False
            
        This is the also default behavior when the value argument is omitted:
            
            >>> delattr(view, 'foo_widget')
            >>> setUpWidget(view, 'foo', IContent['foo'], IFooWidget)
            >>> view.foo_widget.renderedValueSet()
            False
            
        Note that when None is specified for 'value', the widget is configured
        with None:
            
            >>> delattr(view, 'foo_widget')
            >>> setUpWidget(view, 'foo', IContent['foo'], IFooWidget,
            ...             value=None)
            >>> view.foo_widget.renderedValueSet()
            True
            >>> view.foo_widget.getRenderedValue() is None
            True
            
        >>> tearDown()
        """
        
    def test_stickyValues(self):
        """Documents and tests setUpWidget's handling of sticky values.
        
        >>> setUp()
        
        setUpWidget supports the concept of 'sticky values'. A sticky value
        is a value displayed by a widget that should persist across multiple
        across multiple object edit sessions. Sticky values ensure that invalid
        user is available for the user to modify rather than being replaced 
        by some other value.
        
        setUpWidget inferst that a widget has a sticky value if:
            
            - The widget implements IInputWidget
            - The widget returns True for its hasInput method
            
        To illustrate, we'll create and register an edit widget for foo that 
        has input:
            
            >>> class EditFooWidget(Widget):
            ...     implements(IInputWidget)
            ...     _data = "Original Value"
            ...     def hasInput(self): return True
            ...     def getRenderedValue(self): return self._data
            >>> ztapi.browserView(IFoo, '', EditFooWidget, 
            ...                   providing=IInputWidget)
            
        Specifying a value to setUpWidget would typically cause that value
        to be set for the widget:
        
            >>> view = BrowserView(Content(), request)
            >>> setUpWidget(view, 'foo', IContent['foo'], IInputWidget,
            ...     value="A New Value")
            
        However, because EditFooWidget has input (i.e. has a 'sticky' value), 
        setUpWidget will not overwrite its value:
            
            >>> view.foo_widget._data
            'Original Value'
            
        You can use setUpWidget's 'ignoreStickyValues' argument to override
        this behavior and force the widget's value to be overwritten with
        the 'value' argument:
            
            >>> delattr(view, 'foo_widget')
            >>> setUpWidget(view, 'foo', IContent['foo'], IInputWidget,
            ...             value="A New Value", ignoreStickyValues=True)
            >>> view.foo_widget.getRenderedValue()
            'A New Value'
        
        >>> tearDown()
        """

class TestSetUpWidgets(object):
    
    def test_typical(self):
        """Tests the typical use of setUpWidgets.
        
        >>> setUp()
        
        The simplest use of setUpWidget configures a view with widgets of a
        particular type for a schema:
        
            >>> view = BrowserView(Content(), request)
            >>> setUpWidgets(view, IContent, IWidget)
            
        The view now has two widgets, one for each field in the specified
        schema:
            
            >>> IWidget.providedBy(view.foo_widget)
            True
            >>> IWidget.providedBy(view.bar_widget)
            True
            
        Because we did not provide initial values, the widget values are not
        configured:
            
            >>> view.foo_widget.renderedValueSet()
            False
            >>> view.bar_widget.renderedValueSet()
            False
            
        To specify initial values for the widgets, we can use the 'initial'
        argument:
            
            >>> view = BrowserView(Content(), request)
            >>> initial = {'foo': 'Value of Foo', 'bar': 'Value of Bar'}
            >>> setUpWidgets(view, IContent, IWidget, initial=initial)
            >>> view.foo_widget.getRenderedValue()
            'Value of Foo'
            >>> view.bar_widget.getRenderedValue()
            'Value of Bar'
        
        >>> tearDown()
        """        
        
    def test_names(self):
        """Documents and tests the use of names in setUpWidgets.
        
        >>> setUp()
        
        The names argument can be used to configure a specific set of widgets
        for a view:
            
            >>> view = BrowserView(Content(), request)
            >>> IContent.names()
            ['foo', 'bar']
            >>> setUpWidgets(view, IContent, IWidget, names=('bar',))
            >>> hasattr(view, 'foo_widget')
            False
            >>> hasattr(view, 'bar_widget')
            True
        
        >>> tearDown()
        """
        
    def test_delegation(self):
        """Tests setUpWidgets' use of setUpWidget.
        
        >>> setUp()
        
        setUpWidgets delegates several of its arguments to multiple calls to
        setUpWidget - one call for each widget being configured. The arguments
        passed directly through to calls to setUpWidget are:
            
            view
            viewType
            prefix
            ignoreStickyValues
            context

        To illustrate this, we'll replace setUpWidget in the utility module 
        and capture arguments passed to it when setUpWidgets is called.
        
            >>> def setUpWidget(view, name, field, viewType, value=None, 
            ...                 prefix=None, ignoreStickyValues=False, 
            ...                 context=None):
            ...     print "view: %s" % view.__class__
            ...     print "name: %s" % name
            ...     print "field: %s" % field.__class__
            ...     print "viewType: %s" % viewType.__class__
            ...     if value is no_value:
            ...         print "value: not specified"
            ...     else:
            ...         print "value: %s" % value
            ...     print "prefix %s" % prefix
            ...     print "ignoreStickyValues: %s" % ignoreStickyValues
            ...     print "context: %s" % context
            ...     print '---'
            >>> import zope.app.form.utility
            >>> setUpWidgetsSave = zope.app.form.utility.setUpWidget
            >>> zope.app.form.utility.setUpWidget = setUpWidget
            
        When we call setUpWidgets, we should see that setUpWidget is called 
        for each field in the specified schema:
            
            >>> view = BrowserView(Content(), request)
            >>> setUpWidgets(view, IContent, IWidget, 'prefix', True,
            ...              initial={ "bar":"Bar" },
            ...              context="Alt Context")
            view: <class 'zope.publisher.browser.BrowserView'>
            name: foo
            field: <class 'zope.app.form.tests.test_utility.Foo'>
            viewType: <class 'zope.interface.interface.InterfaceClass'>
            value: not specified
            prefix prefix
            ignoreStickyValues: True
            context: Alt Context
            ---
            view: <class 'zope.publisher.browser.BrowserView'>
            name: bar
            field: <class 'zope.app.form.tests.test_utility.Bar'>
            viewType: <class 'zope.interface.interface.InterfaceClass'>
            value: Bar
            prefix prefix
            ignoreStickyValues: True
            context: Alt Context
            ---
            >>> zope.app.form.utility.setUpWidget = setUpWidgetsSave
     
        >>> tearDown()
        """

    def test_forbiddenAttributes(self):
        """Tests that forbidden attributes cause an error in widget setup.

        >>> setUp()

        If an attribute cannot be read from a source object because it's
        forbidden, the ForbiddenAttribute error is allowed to pass through
        to the caller.

        We'll create a field that raises a ForbiddenError itself to simulate
        what would happen when a proxied object's attribute is accessed without
        the required permission.

            >>> class AlwaysForbidden(Field):
            ...     def get(self, source):
            ...         raise ForbiddenAttribute(source, self.__name__)

        We'll also create a schema using this field:

            >>> class IMySchema(Interface):
            ...     tryme = AlwaysForbidden()

        When we use setUpEditWidgets to configure a view with IMySchema:

            >>> view = BrowserView('some context', TestRequest())
            >>> setUpEditWidgets(view, IMySchema)
            Traceback (most recent call last):
            ForbiddenAttribute: ('some context', 'tryme')

        The same applies to setUpDisplayWidgets:

            >>> setUpDisplayWidgets(view, IMySchema)
            Traceback (most recent call last):
            ForbiddenAttribute: ('some context', 'tryme')

        >>> tearDown()
        """
        
class TestFormSetUp(object):
    
    def test_setUpEditWidgets(self):
        """Documents and tests setUpEditWidgets.
        
        >>> setUp()
        
        setUpEditWidgets configures a view to collect field values from a
        user. The function looks up widgets of type IInputWidget for the 
        specified schema.
        
        We'll first create and register widgets for the schema fields for
        which we want input:
            
            >>> class InputWidget(Widget):
            ...     implements(IInputWidget)
            ...     def hasInput(self):
            ...         return False
            ...     def getRenderedValue(self): return self._data
            >>> ztapi.browserViewProviding(IFoo, InputWidget, IInputWidget)
            >>> ztapi.browserViewProviding(IBar, InputWidget, IInputWidget)

        Next we'll configure a view with a context object:
            
            >>> context = Content()
            >>> context.foo = 'abc'
            >>> context.bar = 'def'
            >>> view = BrowserView(context, request)
            
        A call to setUpEditWidgets with the view:
            
            >>> setUpEditWidgets(view, IContent)
            ['foo', 'bar']
            
        configures the view with widgets that accept input for the context 
        field values:
            
            >>> isinstance(view.foo_widget, InputWidget)
            True
            >>> view.foo_widget.getRenderedValue()
            'abc'
            >>> isinstance(view.bar_widget, InputWidget)
            True
            >>> view.bar_widget.getRenderedValue()
            'def'
            
        setUpEditWidgets provides a 'source' argument that provides an
        alternate source of values to be edited:
            
            >>> view = BrowserView(context, request)
            >>> source = Content()
            >>> source.foo = 'abc2'
            >>> source.bar = 'def2'
            >>> setUpEditWidgets(view, IContent, source=source)
            ['foo', 'bar']
            >>> view.foo_widget.getRenderedValue()
            'abc2'
            >>> view.bar_widget.getRenderedValue()
            'def2'
            
        If a field is read only, setUpEditWidgets will use a display widget
        (IDisplayWidget) intead of an input widget to display the field value.
        
            >>> class DisplayWidget(Widget):
            ...     implements(IDisplayWidget)
            >>> ztapi.browserViewProviding(IFoo, DisplayWidget, IDisplayWidget)
            >>> save = IContent['foo'].readonly  # save readonly value
            >>> IContent['foo'].readonly = True
            >>> delattr(view, 'foo_widget')
            >>> setUpEditWidgets(view, IContent)
            ['foo', 'bar']
            >>> isinstance(view.foo_widget, DisplayWidget)
            True
            >>> IContent['foo'].readonly = save  # restore readonly value
        
        By default, setUpEditWidgets raises Unauthorized if it is asked to
        set up a field to which the user does not have permission to
        access or to change.  In the definition of the ExtendedContent
        interface, notice the __Security_checker__ attribute, which stubs
        out a checker that allows the user to view the bar attribute,
        but not set it, and call getAnotherBaz but not setAnotherBaz.
        
            >>> view.context = context = zope.security.checker.Proxy(
            ...     ExtendedContent(), extended_checker)
            >>> setUpEditWidgets(view, IExtendedContent, names=['bar'])
            ... # can' write to bar
            Traceback (most recent call last):
            ...
            Unauthorized: bar
            >>> setUpEditWidgets(
            ... view, IExtendedContent, names=['getAnotherBaz'])
            ... # can't access the setter, setAnotherBaz
            Traceback (most recent call last):
            ...
            Unauthorized: setAnotherBaz
            >>> setUpEditWidgets(
            ... view, IExtendedContent, names=['shazam'])
            ... # can't even access shazam
            Traceback (most recent call last):
            ...
            Unauthorized
        
        Two optional flags can change this behavior.  degradeDisplay=True 
        causes the form machinery to skip fields silently that the user may
        not access.  In this case, the return value of setUpEditWidgets--
        a list of the field names set up--will be different that the names
        provided to the function.
        
            >>> delattr(view, 'foo_widget')
            >>> delattr(view, 'bar_widget')
            >>> ztapi.browserViewProviding(IBaz, InputWidget, IInputWidget)
            >>> setUpEditWidgets(
            ...     view, IExtendedContent, names=['foo', 'shazam', 'getBaz'],
            ...     degradeDisplay=True)
            ['foo', 'getBaz']
            >>> IInputWidget.providedBy(view.foo_widget)
            True
            >>> IInputWidget.providedBy(view.getBaz_widget)
            True
            >>> view.shazam_widget
            Traceback (most recent call last):
            ...
            AttributeError: 'BrowserView' object has no attribute 'shazam_widget'
        
        Similarly, degradeInput=True causes the function to degrade to
        display widgets for any fields that the current user cannot change,
        but can see.
        
            >>> delattr(view, 'foo_widget')
            >>> delattr(view, 'getBaz_widget')
            >>> ztapi.browserViewProviding(IBar, DisplayWidget, IDisplayWidget)
            >>> ztapi.browserViewProviding(IBaz, DisplayWidget, IDisplayWidget)
            >>> setUpEditWidgets(
            ...     view, IExtendedContent, 
            ...     names=['foo', 'bar', 'getBaz', 'getAnotherBaz'],
            ...     degradeInput=True)
            ['foo', 'bar', 'getBaz', 'getAnotherBaz']
            >>> IInputWidget.providedBy(view.foo_widget)
            True
            >>> IDisplayWidget.providedBy(view.bar_widget)
            True
            >>> IInputWidget.providedBy(view.getBaz_widget)
            True
            >>> IDisplayWidget.providedBy(view.getAnotherBaz_widget)
            True
        
        Note that if the user cannot view the current value then they cannot
        view the input widget.  The two flags can then, of course, be used 
        together.
        
            >>> delattr(view, 'foo_widget')
            >>> delattr(view, 'bar_widget')
            >>> delattr(view, 'getBaz_widget')
            >>> delattr(view, 'getAnotherBaz_widget')
            >>> setUpEditWidgets(
            ...     view, IExtendedContent, 
            ...     names=['foo', 'bar', 'shazam', 'getBaz', 'getAnotherBaz'],
            ...     degradeInput=True)
            Traceback (most recent call last):
            ...
            Unauthorized
            >>> setUpEditWidgets(
            ...     view, IExtendedContent, 
            ...     names=['foo', 'bar', 'shazam', 'getBaz', 'getAnotherBaz'],
            ...     degradeInput=True, degradeDisplay=True)
            ['foo', 'bar', 'getBaz', 'getAnotherBaz']
            >>> IInputWidget.providedBy(view.foo_widget)
            True
            >>> IDisplayWidget.providedBy(view.bar_widget)
            True
            >>> IInputWidget.providedBy(view.getBaz_widget)
            True
            >>> IDisplayWidget.providedBy(view.getAnotherBaz_widget)
            True
            >>> view.shazam_widget
            Traceback (most recent call last):
            ...
            AttributeError: 'BrowserView' object has no attribute 'shazam_widget'
        
        >>> tearDown()
        """
        
    def test_setUpDisplayWidgets(self):
        """Documents and tests setUpDisplayWidgets.
        
        >>> setUp()
        
        setUpDisplayWidgets configures a view for use as a display only form.
        The function looks up widgets of type IDisplayWidget for the specified
        schema.
        
        We'll first create and register widgets for the schema fields
        we want to edit:
            
            >>> class DisplayWidget(Widget):
            ...     implements(IDisplayWidget)
            ...     def getRenderedValue(self): return self._data
            >>> ztapi.browserViewProviding(IFoo, DisplayWidget, IDisplayWidget)
            >>> ztapi.browserViewProviding(IBar, DisplayWidget, IDisplayWidget)

        Next we'll configure a view with a context object:
            
            >>> context = Content()
            >>> context.foo = 'abc'
            >>> context.bar = 'def'
            >>> view = BrowserView(context, request)
            
        A call to setUpDisplayWidgets with the view:
            
            >>> setUpDisplayWidgets(view, IContent)
            ['foo', 'bar']
            
        configures the view with widgets that display the context fields:
            
            >>> isinstance(view.foo_widget, DisplayWidget)
            True
            >>> view.foo_widget.getRenderedValue()
            'abc'
            >>> isinstance(view.bar_widget, DisplayWidget)
            True
            >>> view.bar_widget.getRenderedValue()
            'def'
            
        Like setUpEditWidgets, setUpDisplayWidgets accepts a 'source'
        argument that provides an alternate source of values to be edited:
            
            >>> view = BrowserView(context, request)
            >>> source = Content()
            >>> source.foo = 'abc2'
            >>> source.bar = 'def2'
            >>> setUpDisplayWidgets(view, IContent, source=source)
            ['foo', 'bar']
            >>> view.foo_widget.getRenderedValue()
            'abc2'
            >>> view.bar_widget.getRenderedValue()
            'def2'
        
        Also like setUpEditWidgets, the degradeDisplay flag allows widgets
        to silently disappear if they are unavailable.
        
            >>> view.context = context = zope.security.checker.Proxy(
            ...     ExtendedContent(), extended_checker)
            >>> delattr(view, 'foo_widget')
            >>> delattr(view, 'bar_widget')
            >>> ztapi.browserViewProviding(IBaz, DisplayWidget, IDisplayWidget)
            >>> setUpDisplayWidgets(
            ...     view, IExtendedContent, 
            ...     names=['foo', 'bar', 'shazam', 'getBaz', 'getAnotherBaz'])
            Traceback (most recent call last):
            ...
            Unauthorized
            >>> setUpDisplayWidgets(
            ...     view, IExtendedContent, 
            ...     names=['foo', 'bar', 'shazam', 'getBaz', 'getAnotherBaz'],
            ...     degradeDisplay=True)
            ['foo', 'bar', 'getBaz', 'getAnotherBaz']
            >>> IDisplayWidget.providedBy(view.foo_widget)
            True
            >>> IDisplayWidget.providedBy(view.bar_widget)
            True
            >>> IDisplayWidget.providedBy(view.getBaz_widget)
            True
            >>> IDisplayWidget.providedBy(view.getAnotherBaz_widget)
            True
            >>> view.shazam_widget
            Traceback (most recent call last):
            ...
            AttributeError: 'BrowserView' object has no attribute 'shazam_widget'
        
        >>> tearDown()
        """
        
class TestForms(object):
    
    def test_viewHasInput(self):
        """Tests viewHasInput.
        
        >>> setUp()
        
        viewHasInput returns True if any of the widgets for a set of fields
        have user input.
        
        This method is typically invoked on a view that has been configured
        with one setUpEditWidgets.
        
            >>> class InputWidget(Widget):
            ...     implements(IInputWidget)
            ...     input = None
            ...     def hasInput(self):
            ...         return self.input is not None
            >>> ztapi.browserViewProviding(IFoo, InputWidget, IInputWidget)
            >>> ztapi.browserViewProviding(IBar, InputWidget, IInputWidget)
            >>> view = BrowserView(Content(), request)
            >>> setUpEditWidgets(view, IContent)
            ['foo', 'bar']
            
        Because InputWidget is configured to not have input by default, the
        view does not have input:
            
            >>> viewHasInput(view, IContent)
            False
            
        But if we specify input for at least one widget:
            
            >>> view.foo_widget.input = 'Some Value'
            >>> viewHasInput(view, IContent)
            True
            
        >>> tearDown()
        """
        
    def test_applyWidgetsChanges(self):
        """Documents and tests applyWidgetsChanges.
        
        >>> setUp()
        
        applyWidgetsChanges updates the view context, or an optional alternate
        context, with widget values. This is typically called when a form
        is submitted.
        
        We'll first create a simple edit widget that can be used to update
        an object:
        
            >>> class InputWidget(Widget):
            ...     implements(IInputWidget)           
            ...     input = None
            ...     valid = True
            ...     def hasInput(self):
            ...         return input is not None
            ...     def applyChanges(self, object):
            ...         if not self.valid:
            ...             raise ConversionError('invalid input')
            ...         field = self.context
            ...         field.set(object, self.input)
            ...         return True
            >>> ztapi.browserViewProviding(IFoo, InputWidget, IInputWidget)
            >>> ztapi.browserViewProviding(IBar, InputWidget, IInputWidget)
            
        Before calling applyWidgetsUpdate, we need to configure a context and
        a view with edit widgets:
            
            >>> context = Content()
            >>> view = BrowserView(context, request)
            >>> setUpEditWidgets(
            ...     view, IContent, context=context, names=('foo',))
            ['foo']
            
        We now specify new widget input and apply the changes: 

            >>> view.foo_widget.input = 'The quick brown fox...'
            >>> context.foo
            'Foo'
            >>> applyWidgetsChanges(view, IContent, names=('foo',))
            True
            >>> context.foo
            'The quick brown fox...'
            
        By default, applyWidgetsChanges applies the new widget values to the
        view context. Alternatively, we can provide a 'target' argument to
        be updated:
            
            >>> target = Content()
            >>> target.foo
            'Foo'
            >>> applyWidgetsChanges(view, IContent, target=target, 
            ...                     names=('foo',))
            True
            >>> target.foo
            'The quick brown fox...'
            
        applyWidgetsChanges is typically used in conjunction with one of the
        setUp utility functions. If applyWidgetsChanges is called using a
        view that was not previously configured with a setUp function, or
        was not otherwise configured with widgets for each of the applicable
        fields, an AttributeError will be raised:
            
            >>> view = BrowserView(context, request)
            >>> applyWidgetsChanges(view, IContent, names=('foo',))
            Traceback (most recent call last):
            AttributeError: 'BrowserView' object has no attribute 'foo_widget'

        When applyWidgetsChanges is called with multiple form
        fields, some with valid data and some with invalid data, 
        *changes may be applied*.  For instance, below see that context.foo
        changes from 'Foo' to 'a' even though trying to change context.bar
        fails.  Generally, ZODB transactional behavior is expected to
        correct this sort of problem.

            >>> context = Content()
            >>> view = BrowserView(context, request)
            >>> setUpEditWidgets(view, IContent, names=('foo', 'bar'))
            ['foo', 'bar']
            >>> view.foo_widget.input = 'a'
            >>> view.bar_widget.input = 'b'
            >>> view.bar_widget.valid = False
            >>> context.foo
            'Foo'
            >>> getattr(context, 'bar', 'not really')
            'not really'
            >>> applyWidgetsChanges(view, IContent, names=('foo', 'bar'))
            Traceback (most recent call last):
            WidgetsError: ConversionError: ('invalid input', None)
            >>> context.foo
            'a'
            >>> getattr(context, 'bar', 'not really')
            'not really'

        >>> tearDown()
        """
        
class TestGetWidgetsData(object):
    
    def test_typical(self):
        """Documents and tests the typical use of getWidgetsData.
        
        >>> setUp()
        
        getWidgetsData retrieves the current values from widgets on a view.
        For this test, we'll create a simple edit widget and register it
        for the schema field types:
            
            >>> class InputWidget(Widget):
            ...     implements(IInputWidget)
            ...     input = None
            ...     def hasInput(self):
            ...         return self.input is not None
            ...     def getInputValue(self):
            ...         return self.input
            >>> ztapi.browserViewProviding(IFoo, InputWidget, IInputWidget)
            >>> ztapi.browserViewProviding(IBar, InputWidget, IInputWidget)

        We use setUpEditWidgets to configure a view with widgets for the
        IContent schema:
        
            >>> view = BrowserView(Content(), request)
            >>> setUpEditWidgets(view, IContent)
            ['foo', 'bar']
            
        The simplest form of getWidgetsData requires a view and a schema:
            
            >>> try:
            ...     result = getWidgetsData(view, IContent)
            ... except Exception, e:
            ...     print 'getWidgetsData failed'
            ...     e
            getWidgetsData failed
            MissingInputError: ('foo', u'', 'the field is required')
            MissingInputError: ('bar', u'', 'the field is required')
            
        We see that getWidgetsData raises a MissingInputError if a required
        field does not have input from a widget.:
            
            >>> view.foo_widget.input = 'Input for foo'
            >>> view.bar_widget.input = 'Input for bar'
            >>> result = getWidgetsData(view, IContent)
            
        The result of getWidgetsData is a map of field names to widget values.
        
            >>> keys = result.keys(); keys.sort(); keys
            ['bar', 'foo']
            >>> result['foo']
            'Input for foo'
            >>> result['bar']
            'Input for bar'
            
        If a field is not required, however:
            
            >>> IContent['foo'].required = False
            
        we can omit input for its widget:
            
            >>> view.foo_widget.input = None
            >>> result = getWidgetsData(view, IContent)
            >>> 'foo' in result
            False
            
        Note that when a widget indicates that is does not have input, its
        results are omitted from getWidgetsData's return value. Users of
        getWidgetsData should explicitly check for field values before
        accessing them:
            
            >>> for name in IContent:
            ...     if name in result:
            ...         print (name, result[name])
            ('bar', 'Input for bar')
            
        You can also specify an optional 'names' argument (a tuple) to 
        request a subset of the schema fields:
            
            >>> result = getWidgetsData(view, IContent, names=('bar',))
            >>> result.keys()
            ['bar']
            
        >>> tearDown()
        """
        
    def test_widgetsErrorException(self):
        """Documents and tests WidgetsError.
        
        WidgetsError wraps one or more errors, which are specified as a
        sequence in the 'errors' argument:
        
            >>> error = WidgetsError(('foo',))
            >>> error
            str: foo
            
        WidgetsError also provides a 'widgetsData' attribute, which is a
        map of valid field values, keyed by field name, that were obtained
        in the same read operation that generated the errors:
            
            >>> error = WidgetsError(('foo',), widgetsData={'bar': 'Bar'})
            >>> error.widgetsData
            {'bar': 'Bar'}
            
        The most typical use of this error is when reading a set of widget
        values -- the read operation can generate more than one error, as well
        as a set of successfully read values:
            
            >>> values = {'foo': 'Foo'}
            >>> errors = []
            >>> widgetsData = {}
            >>> for name in ('foo', 'bar'):  # some list of values to read
            ...     try:
            ...         widgetsData[name] = values[name]  # read operation
            ...     except Exception, e:
            ...         errors.append(e)    # capture all errors
            >>> if errors:
            ...     widgetsError = WidgetsError(errors, widgetsData)
            ...     raise widgetsError
            Traceback (most recent call last):
            WidgetsError: KeyError: 'bar'
            
        The handler of error can access all of the widget error as well as
        the widget values read:
            
            >>> for error in widgetsError:
            ...     error.__class__.__name__
            'KeyError'
            >>> widgetsError.widgetsData
            {'foo': 'Foo'}
        """
            
def test_suite():
    return doctest.DocTestSuite()

if __name__=='__main__':
    main(defaultTest='test_suite')



##############################################################################
#
# Copyright (c) 2004 Zope Corporation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Source Widget Tests

$Id: test_source.py 70826 2006-10-20 03:41:16Z baijum $
"""
from zope.app.testing import placelesssetup

def test_suite():
    from zope.testing import doctest
    return doctest.DocFileSuite(
        '../source.txt',
        setUp=placelesssetup.setUp, tearDown=placelesssetup.tearDown)

if __name__ == '__main__':
    import unittest
    unittest.main(defaultTest='test_suite')

